@@ -251,12 +251,6 @@
 /* If so, is it restricted to user and system time? */
 #undef GETRUSAGE_RESTRICTED
 
-/* Solaris has rusage as an ioctl on procfs */
-#undef GETRUSAGE_THROUGH_PROCFS
-
-/* So has True64, but no useful information in prstatus_t */
-#undef GETRUSAGE_THROUGH_PROCFS_PRS
-
 /* Define if you have infnan */
 #undef HAVE_INFNAN
 
@@ -413,6 +407,9 @@
 /* define this if #include <time.h> provides an external int timezone */
 #undef HAVE_EXTERNAL_TIMEZONE
 
+/* define this if #include <time.h> provides an external int altzone */
+#undef HAVE_EXTERNAL_ALTZONE
+
 /* define this if your struct tm has a tm_gmtoff */
 #undef STRUCT_TM_HAS_GMTOFF
 
