@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: ec525e2bb7f0ea611222dc01ae757029ec0658f6 $
 */
 
 #undef BLOCK_ALLOC
@@ -22,11 +21,8 @@
 
 #define BLOCK_ALLOC(DATA,SIZE)						\
 struct DATA *PIKE_CONCAT(alloc_,DATA)(void);				\
-void PIKE_CONCAT3(new_,DATA,_context)(void);				\
-void PIKE_CONCAT(really_free_,DATA)(struct DATA *d);			\
-void PIKE_CONCAT3(free_all_,DATA,_blocks)(void);			\
-void PIKE_CONCAT3(count_memory_in_,DATA,s)(size_t *num, size_t *size);	\
-void PIKE_CONCAT3(init_,DATA,_blocks)(void)
+PMOD_EXPORT void PIKE_CONCAT(really_free_,DATA)(struct DATA *d);			\
+void PIKE_CONCAT3(count_memory_in_,DATA,s)(size_t *num, size_t *size);
 
 
 #define PTR_HASH_ALLOC(DATA,BSIZE)				\
@@ -34,16 +30,11 @@ BLOCK_ALLOC(DATA,BSIZE);					\
 extern struct DATA **PIKE_CONCAT(DATA,_hash_table);		\
 extern size_t PIKE_CONCAT(DATA,_hash_table_size);		\
 struct DATA *PIKE_CONCAT(find_,DATA)(void *ptr);		\
-struct DATA *PIKE_CONCAT3(make_,DATA,_unlocked)			\
-		(void *ptr, PIKE_HASH_T hval);			\
 struct DATA *PIKE_CONCAT(make_,DATA)(void *ptr);		\
 struct DATA *PIKE_CONCAT(get_,DATA)(void *ptr);			\
 int PIKE_CONCAT3(check_,DATA,_semaphore)(void *ptr);		\
 void PIKE_CONCAT(move_,DATA)(struct DATA *block, void *new_ptr); \
-int PIKE_CONCAT(remove_,DATA)(void *ptr);			\
-void PIKE_CONCAT3(low_init_,DATA,_hash)(size_t);		\
-void PIKE_CONCAT3(init_,DATA,_hash)(void);			\
-void PIKE_CONCAT3(exit_,DATA,_hash)(void)
+int PIKE_CONCAT(remove_,DATA)(void *ptr);
 
 #define PTR_HASH_ALLOC_FIXED(DATA,BSIZE)			\
 PTR_HASH_ALLOC(DATA,BSIZE)
