@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 0d62f9b03261f5ffdf0199623b47d8b8dfc70be9 $
 */
 
 /*
@@ -235,7 +234,7 @@
  * I6	Frame pointer
  * I7	Return address
  *
- * L0	&Pike_interpreter
+ * L0	Pike_interpreter_pointer
  * L1	Pike_fp
  * L7	&d_flag
  *
@@ -271,7 +270,8 @@ ptrdiff_t sparc_last_pc = 0;
 #define LOAD_PIKE_INTERPRETER() do {				\
     if (!(sparc_codegen_state & SPARC_CODEGEN_IP_IS_SET)) {	\
       SET_REG(SPARC_REG_PIKE_IP,				\
-	      ((ptrdiff_t)(&Pike_interpreter)));		\
+	      ((ptrdiff_t)(&Pike_interpreter_pointer)));	\
+      PIKE_LDPTR(SPARC_REG_PIKE_IP, SPARC_REG_PIKE_IP, 0, 1);	\
       sparc_codegen_state |= SPARC_CODEGEN_IP_IS_SET;		\
     }								\
   } while(0)
@@ -281,7 +281,7 @@ ptrdiff_t sparc_last_pc = 0;
       LOAD_PIKE_INTERPRETER();						\
       /* lduw [ %ip, %offset(Pike_interpreter, frame_pointer) ], %l1 */	\
       PIKE_LDPTR(SPARC_REG_PIKE_FP, SPARC_REG_PIKE_IP,			\
-		 OFFSETOF(Pike_interpreter, frame_pointer), 1);		\
+		 OFFSETOF(Pike_interpreter_struct, frame_pointer), 1);	\
       sparc_codegen_state |= SPARC_CODEGEN_FP_IS_SET;			\
     }									\
   } while(0)
@@ -291,7 +291,7 @@ ptrdiff_t sparc_last_pc = 0;
       LOAD_PIKE_INTERPRETER();						\
       /* lduw [ %ip, %offset(Pike_interpreter, stack_pointer) ], %l2 */	\
       PIKE_LDPTR(SPARC_REG_PIKE_SP, SPARC_REG_PIKE_IP,			\
-		 OFFSETOF(Pike_interpreter, stack_pointer), 1);		\
+		 OFFSETOF(Pike_interpreter_struct, stack_pointer), 1);	\
       sparc_codegen_state |= SPARC_CODEGEN_SP_IS_SET;			\
       sparc_pike_sp_bias = 0;						\
     } else if (sparc_pike_sp_bias > 0xf00) {				\
@@ -308,7 +308,7 @@ ptrdiff_t sparc_last_pc = 0;
       LOAD_PIKE_INTERPRETER();						\
       /* lduw [ %ip, %offset(Pike_interpreter, mark_stack_pointer) ], %l2 */	\
       PIKE_LDPTR(SPARC_REG_PIKE_MARK_SP, SPARC_REG_PIKE_IP,		\
-		 OFFSETOF(Pike_interpreter, mark_stack_pointer), 1);	\
+		 OFFSETOF(Pike_interpreter_struct, mark_stack_pointer), 1);	\
       sparc_codegen_state |= SPARC_CODEGEN_MARK_SP_IS_SET;		\
     }									\
   } while(0)
@@ -323,13 +323,13 @@ ptrdiff_t sparc_last_pc = 0;
     if (sparc_codegen_state & SPARC_CODEGEN_MARK_SP_NEEDS_STORE) {	\
       /* stw %pike_mark_sp, [ %ip, %offset(Pike_interpreter, mark_stack_pointer) ] */	\
       PIKE_STPTR(SPARC_REG_PIKE_MARK_SP, SPARC_REG_PIKE_IP,		\
-		 OFFSETOF(Pike_interpreter, mark_stack_pointer), 1);	\
+		 OFFSETOF(Pike_interpreter_struct, mark_stack_pointer), 1);	\
       sparc_codegen_state &= ~SPARC_CODEGEN_MARK_SP_NEEDS_STORE;	\
     }									\
     if (sparc_codegen_state & SPARC_CODEGEN_SP_NEEDS_STORE) {		\
       /* stw %pike_sp, [ %ip, %offset(Pike_interpreter, stack_pointer) ] */	\
       PIKE_STPTR(SPARC_REG_PIKE_SP, SPARC_REG_PIKE_IP,			\
-		 OFFSETOF(Pike_interpreter, stack_pointer), 1);		\
+		 OFFSETOF(Pike_interpreter_struct, stack_pointer), 1);	\
       sparc_codegen_state &= ~SPARC_CODEGEN_SP_NEEDS_STORE;		\
     }									\
   } while(0)
@@ -477,12 +477,14 @@ static void sparc_push_int(INT_TYPE x, int sub_type)
     SPARC_STX(reg, SPARC_REG_PIKE_SP,
 	      sparc_pike_sp_bias + OFFSETOF(svalue, u.integer), 1);
   }
-  if (x != type_word) {
+  if (!type_word) {
+    reg = SPARC_REG_G0;
+  } else if (x != type_word) {
     SET_REG(SPARC_REG_I1, type_word);
+    reg = SPARC_REG_I1;
   }
-  /* This is safe since type_word is never zero. */
-  /* stw %i1, [ %pike_sp ] */
-  SPARC_STW(SPARC_REG_I1, SPARC_REG_PIKE_SP, sparc_pike_sp_bias, 1);
+  /* stw %reg, [ %pike_sp ] */
+  SPARC_STW(reg, SPARC_REG_PIKE_SP, sparc_pike_sp_bias, 1);
   sparc_pike_sp_bias += sizeof(struct svalue);
   sparc_codegen_state |= SPARC_CODEGEN_SP_NEEDS_STORE;
 }
@@ -492,14 +494,14 @@ static void sparc_clear_string_subtype(void)
   LOAD_PIKE_SP();
   /* lduh [ %pike_sp, %g0 ], %i0 */
   SPARC_LDUH(SPARC_REG_I0, SPARC_REG_PIKE_SP,
-	     sparc_pike_sp_bias + OFFSETOF(svalue, type), 1);
+	     sparc_pike_sp_bias + OFFSETOF(svalue, tu.t.type), 1);
   /* subcc %g0, %i0, 8 */
   SPARC_SUBcc(SPARC_REG_G0, SPARC_REG_I0, PIKE_T_INT, 1);
   /* be,a .+8 */
   SPARC_BE(8, 1);
   /* sth %g0, [ %pike_sp, 2 ] */
   SPARC_STH(SPARC_REG_G0, SPARC_REG_PIKE_SP,
-	    sparc_pike_sp_bias + OFFSETOF(svalue, subtype), 1);
+	    sparc_pike_sp_bias + OFFSETOF(svalue, tu.t.subtype), 1);
 }
 
 static void sparc_push_lfun(unsigned int no)
@@ -574,13 +576,13 @@ void sparc_local_lvalue(unsigned int no)
 
 #ifdef PIKE_DEBUG
 void sparc_debug_check_registers(int state,
-				 struct Pike_interpreter *cached_ip,
+				 struct Pike_interpreter_struct *cached_ip,
 				 struct pike_frame *cached_fp,
 				 struct svalue *cached_sp,
 				 struct svalue **cached_mark_sp)
 {
   if (((state & SPARC_CODEGEN_IP_IS_SET) &&
-       (cached_ip != &Pike_interpreter)) ||
+       (cached_ip != Pike_interpreter_pointer)) ||
       ((state & SPARC_CODEGEN_FP_IS_SET) &&
        (cached_fp != Pike_interpreter.frame_pointer)) ||
       ((state & SPARC_CODEGEN_SP_IS_SET) &&
@@ -593,7 +595,8 @@ void sparc_debug_check_registers(int state,
 	       state,
 	       (INT32)cached_ip, (INT32)cached_fp,
 	       (INT32)cached_sp, (INT32)cached_mark_sp,
-	       (INT32)&Pike_interpreter, (INT32)Pike_interpreter.frame_pointer,
+	       (INT32)Pike_interpreter_pointer,
+	       (INT32)Pike_interpreter.frame_pointer,
 	       (INT32)Pike_interpreter.stack_pointer,
 	       (INT32)Pike_interpreter.mark_stack_pointer);
   }
@@ -642,6 +645,7 @@ static void ins_sparc_debug()
 
 static void low_ins_call(void *addr, int delay_ok, int i_flags)
 {
+  LOAD_PIKE_INTERPRETER();
   SPARC_FLUSH_UNSTORED();
 
   {
