@@ -306,14 +306,17 @@ int do_docode(node *n, int flags)
   int stack_depth_save = current_stack_depth;
   struct compilation *c = THIS_COMPILATION;
   INT_TYPE save_current_line = c->lex.current_line;
+  struct pike_string *save_current_file = c->lex.current_file;
   if(!n) return 0;
   c->lex.current_line=n->line_number;
+  c->lex.current_file = n->current_file;
 #ifdef PIKE_DEBUG
   if (current_stack_depth == -4711) Pike_fatal("do_docode() used outside docode().\n");
 #endif
   i=do_docode2(n, flags);
   current_stack_depth = stack_depth_save + i;
 
+  c->lex.current_file = save_current_file;
   c->lex.current_line=save_current_line;
   return i;
 }
@@ -617,9 +620,11 @@ static int do_encode_automap_arg_list(node *n,
 	depth++;
       }
       emit0(F_MARK);
+      PUSH_CLEANUP_FRAME(do_pop_mark, 0);
       code_expression(n, 0, "[*]");
       emit1(F_NUMBER, depth);
       emit_apply_builtin("__builtin.automap_marker");
+      POP_AND_DONT_CLEANUP;
       return 1;
     }
   }
@@ -896,9 +901,11 @@ static void emit_multi_assign(node *vals, node *vars, int no)
 	} else if (!level) {
 	  f += inh->identifier_level;
 	  emit0(F_MARK);
+	  PUSH_CLEANUP_FRAME(do_pop_mark, 0);
 	  code_expression(val, 0, "RHS");
 	  emit_multi_assign(vals, vars, no+1);
 	  emit1(F_CALL_LFUN, f);
+	  POP_AND_DONT_CLEANUP;
 	  emit0(F_POP_VALUE);
 	}
       }
@@ -1427,6 +1434,7 @@ static int do_docode2(node *n, int flags)
             yywarning("Variable %S lacks a setter.", id->name);
           } else if (!level) {
             f += inh->identifier_level;
+	    PUSH_CLEANUP_FRAME(do_pop_mark, 0);
             if (flags & DO_POP) {
 #ifndef USE_APPLY_N
               emit0(F_MARK);
@@ -1444,6 +1452,7 @@ static int do_docode2(node *n, int flags)
 #else
             emit1(F_CALL_LFUN, f);
 #endif
+	    POP_AND_DONT_CLEANUP;
             emit0(F_POP_VALUE);
             return !(flags & DO_POP);
           }
@@ -1538,15 +1547,19 @@ static int do_docode2(node *n, int flags)
 	Pike_fatal("HELP! FATAL INTERNAL COMPILER ERROR (1)\n");
 #endif
 
+      PUSH_CLEANUP_FRAME(do_pop_mark, 0);
       emit0(F_MARK);
+      PUSH_CLEANUP_FRAME(do_pop_mark, 0);
       emit0(F_MARK);
       emit0(F_LTOSVAL);
       emit1(F_NUMBER, depth);
       emit_apply_builtin("__builtin.automap_marker");
+      POP_AND_DONT_CLEANUP;
       emit_builtin_svalue("`+");
       emit2(F_REARRANGE,1,1);
       emit1(F_NUMBER, 1);
       emit_apply_builtin("__automap__");
+      POP_AND_DONT_CLEANUP;
 
       if(flags & DO_POP)
       {
@@ -1600,15 +1613,19 @@ static int do_docode2(node *n, int flags)
 	Pike_fatal("HELP! FATAL INTERNAL COMPILER ERROR (1)\n");
 #endif
 
+      PUSH_CLEANUP_FRAME(do_pop_mark, 0);
       emit0(F_MARK);
+      PUSH_CLEANUP_FRAME(do_pop_mark, 0);
       emit0(F_MARK);
       emit0(F_LTOSVAL);
       emit1(F_NUMBER, depth);
       emit_apply_builtin("__builtin.automap_marker");
+      POP_AND_DONT_CLEANUP;
       emit_builtin_svalue("`-");
       emit2(F_REARRANGE,1,1);
       emit1(F_NUMBER, 1);
       emit_apply_builtin("__automap__");
+      POP_AND_DONT_CLEANUP;
 
       if(flags & DO_POP)
       {
@@ -2758,9 +2775,11 @@ static int do_docode2(node *n, int flags)
 
   case F_AUTO_MAP:
     emit0(F_MARK);
+    PUSH_CLEANUP_FRAME(do_pop_mark, 0);
     code_expression(CAR(n), 0, "automap function");
     do_encode_automap_arg_list(CDR(n),0);
     emit_apply_builtin("__automap__");
+    POP_AND_DONT_CLEANUP;
     return 1;
 
   case F_AUTO_MAP_MARKER:
