@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 0520c8c6b3dfcb18e9951f9f67593b4e4c73a5f2 $
 */
 
 #include "global.h"
@@ -28,9 +27,9 @@ PMOD_EXPORT char *low_make_buf_space(ptrdiff_t space, dynamic_buffer *buf)
       buf->bufsize*=2;
     }while(buf->s.len+space >= buf->bufsize);
 
-    buf->s.str=(char *)realloc(buf->s.str, buf->bufsize);
+    buf->s.str=realloc(buf->s.str, buf->bufsize);
     if(!buf->s.str)
-      Pike_error("Out of memory.\n");
+      Pike_fatal("Out of memory.\n");
   }
   ret = buf->s.str + buf->s.len;
   buf->s.len += space;
@@ -54,12 +53,12 @@ PMOD_EXPORT void low_my_binary_strcat(const char *b, size_t l,
     Pike_fatal("Error in internal buffering.\n");
 #endif
 
-  MEMCPY(low_make_buf_space(l, buf),b, l);
+  memcpy(low_make_buf_space(l, buf),b, l);
 }
 
 PMOD_EXPORT void debug_initialize_buf(dynamic_buffer *buf)
 {
-  buf->s.str=(char *)xalloc((buf->bufsize=BUFFER_BEGIN_SIZE));
+  buf->s.str=xalloc((buf->bufsize=BUFFER_BEGIN_SIZE));
   *(buf->s.str)=0;
   buf->s.len=0;
 }
