@@ -14,6 +14,7 @@
 
 #ifndef _LARGEFILE_SOURCE
 #  define _FILE_OFFSET_BITS 64
+#  define _TIME_BITS 64
 #  define _LARGEFILE_SOURCE
 /* #  define _LARGEFILE64_SOURCE 1 */	/* This one is for explicit 64bit. */
 #endif
@@ -602,7 +603,7 @@ typedef struct p_wchar_p
  * themselves, unless they are compiled statically. */
 #   define PMOD_EXPORT __declspec(dllexport)
 #  endif
-# elif defined(__clang__) && defined(MAC_OS_X_VERSION_MIN_REQUIRED)
+# elif defined(__clang__) && (defined(MAC_OS_X_VERSION_MIN_REQUIRED) || defined(__ENVIRONMENT_MAC_OS_X_VERSION_MIN_REQUIRED__))
 /* According to Clang source the protected behavior is ELF-specific and not
    applicable to OS X. */
 #  define PMOD_EXPORT    __attribute__ ((visibility("default")))
