@@ -221,11 +221,10 @@ PMOD_EXPORT extern const char msg_pop_neg[];
  do {									\
    ptrdiff_t x_=(X);							\
    if(x_) {								\
-     struct svalue *_sp_;						\
      check__positive(x_, (msg_pop_neg, x_));				\
-     _sp_ = Pike_sp = Pike_sp - x_;					\
+     if (x_ > 131072) Pike_fatal("Popping too much!\n");		\
+     while (x_--) pop_stack();						\
      debug_check_stack();						\
-     free_mixed_svalues(_sp_, x_);					\
    }									\
  } while (0)
 
@@ -708,7 +707,7 @@ PMOD_EXPORT extern unsigned long evaluator_callback_calls;
     low_check_threads_etc();						\
   } while (0)
 
-extern int fast_check_threads_counter;
+PMOD_EXPORT extern int fast_check_threads_counter;
 
 #define fast_check_threads_etc(X) do {					\
     DO_IF_DEBUG (if (Pike_interpreter.trace_level > 2)			\
