@@ -1,14 +1,13 @@
-/* A Bison parser, made by GNU Bison 2.3.  */
+/* A Bison parser, made by GNU Bison 3.0.2.  */
 
-/* Skeleton interface for Bison's Yacc-like parsers in C
+/* Bison interface for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
-   Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989-1990, 2000-2013 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -16,9 +15,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* As a special exception, you may create a larger work that contains
    part or all of the Bison parser skeleton and distribute that work
@@ -33,100 +30,109 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-/* Tokens.  */
+#ifndef YY_YY_Y_TAB_H_INCLUDED
+# define YY_YY_Y_TAB_H_INCLUDED
+/* Debug traces.  */
+#ifndef YYDEBUG
+# define YYDEBUG 0
+#endif
+#if YYDEBUG
+extern int yydebug;
+#endif
+
+/* Token type.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
-   /* Put the tokens into the symbol table, so that GDB and other debuggers
-      know about them.  */
-   enum yytokentype {
-     TOK_ARROW = 258,
-     TOK_CONSTANT = 259,
-     TOK_FLOAT = 260,
-     TOK_STRING = 261,
-     TOK_NUMBER = 262,
-     TOK_INC = 263,
-     TOK_DEC = 264,
-     TOK_RETURN = 265,
-     TOK_EQ = 266,
-     TOK_GE = 267,
-     TOK_LE = 268,
-     TOK_NE = 269,
-     TOK_NOT = 270,
-     TOK_LSH = 271,
-     TOK_RSH = 272,
-     TOK_LAND = 273,
-     TOK_LOR = 274,
-     TOK_SWITCH = 275,
-     TOK_SSCANF = 276,
-     TOK_CATCH = 277,
-     TOK_FOREACH = 278,
-     TOK_LEX_EOF = 279,
-     TOK_ADD_EQ = 280,
-     TOK_AND_EQ = 281,
-     TOK_ARRAY_ID = 282,
-     TOK_ATTRIBUTE_ID = 283,
-     TOK_BREAK = 284,
-     TOK_CASE = 285,
-     TOK_CLASS = 286,
-     TOK_COLON_COLON = 287,
-     TOK_CONTINUE = 288,
-     TOK_DEFAULT = 289,
-     TOK_DEPRECATED_ID = 290,
-     TOK_DIV_EQ = 291,
-     TOK_DO = 292,
-     TOK_DOT_DOT = 293,
-     TOK_DOT_DOT_DOT = 294,
-     TOK_ELSE = 295,
-     TOK_ENUM = 296,
-     TOK_EXTERN = 297,
-     TOK_FLOAT_ID = 298,
-     TOK_FOR = 299,
-     TOK_FUNCTION_ID = 300,
-     TOK_GAUGE = 301,
-     TOK_GLOBAL = 302,
-     TOK_IDENTIFIER = 303,
-     TOK_RESERVED = 304,
-     TOK_IF = 305,
-     TOK_IMPORT = 306,
-     TOK_INHERIT = 307,
-     TOK_INLINE = 308,
-     TOK_LOCAL_ID = 309,
-     TOK_FINAL_ID = 310,
-     TOK_FUNCTION_NAME = 311,
-     TOK_INT_ID = 312,
-     TOK_LAMBDA = 313,
-     TOK_MULTISET_ID = 314,
-     TOK_MULTISET_END = 315,
-     TOK_MULTISET_START = 316,
-     TOK_LSH_EQ = 317,
-     TOK_MAPPING_ID = 318,
-     TOK_MIXED_ID = 319,
-     TOK_MOD_EQ = 320,
-     TOK_MULT_EQ = 321,
-     TOK_NO_MASK = 322,
-     TOK_OBJECT_ID = 323,
-     TOK_OR_EQ = 324,
-     TOK_PRIVATE = 325,
-     TOK_PROGRAM_ID = 326,
-     TOK_PROTECTED = 327,
-     TOK_PREDEF = 328,
-     TOK_PUBLIC = 329,
-     TOK_RSH_EQ = 330,
-     TOK_STATIC = 331,
-     TOK_STRING_ID = 332,
-     TOK_SUB_EQ = 333,
-     TOK_TYPEDEF = 334,
-     TOK_TYPEOF = 335,
-     TOK_VARIANT = 336,
-     TOK_VERSION = 337,
-     TOK_VOID_ID = 338,
-     TOK_WHILE = 339,
-     TOK_XOR_EQ = 340,
-     TOK_OPTIONAL = 341,
-     TOK_SAFE_INDEX = 342,
-     TOK_SAFE_START_INDEX = 343,
-     TOK_BITS = 344
-   };
+  enum yytokentype
+  {
+    TOK_ARROW = 258,
+    TOK_CONSTANT = 259,
+    TOK_FLOAT = 260,
+    TOK_STRING = 261,
+    TOK_NUMBER = 262,
+    TOK_INC = 263,
+    TOK_DEC = 264,
+    TOK_RETURN = 265,
+    TOK_EQ = 266,
+    TOK_GE = 267,
+    TOK_LE = 268,
+    TOK_NE = 269,
+    TOK_NOT = 270,
+    TOK_LSH = 271,
+    TOK_RSH = 272,
+    TOK_LAND = 273,
+    TOK_LOR = 274,
+    TOK_SWITCH = 275,
+    TOK_SSCANF = 276,
+    TOK_CATCH = 277,
+    TOK_FOREACH = 278,
+    TOK_LEX_EOF = 279,
+    TOK_ADD_EQ = 280,
+    TOK_AND_EQ = 281,
+    TOK_ARRAY_ID = 282,
+    TOK_ATTRIBUTE_ID = 283,
+    TOK_BREAK = 284,
+    TOK_CASE = 285,
+    TOK_CLASS = 286,
+    TOK_COLON_COLON = 287,
+    TOK_CONTINUE = 288,
+    TOK_DEFAULT = 289,
+    TOK_DEPRECATED_ID = 290,
+    TOK_DIV_EQ = 291,
+    TOK_DO = 292,
+    TOK_DOT_DOT = 293,
+    TOK_DOT_DOT_DOT = 294,
+    TOK_ELSE = 295,
+    TOK_ENUM = 296,
+    TOK_EXTERN = 297,
+    TOK_FLOAT_ID = 298,
+    TOK_FOR = 299,
+    TOK_FUNCTION_ID = 300,
+    TOK_GAUGE = 301,
+    TOK_GLOBAL = 302,
+    TOK_IDENTIFIER = 303,
+    TOK_RESERVED = 304,
+    TOK_IF = 305,
+    TOK_IMPORT = 306,
+    TOK_INHERIT = 307,
+    TOK_INLINE = 308,
+    TOK_LOCAL_ID = 309,
+    TOK_FINAL_ID = 310,
+    TOK_FUNCTION_NAME = 311,
+    TOK_INT_ID = 312,
+    TOK_LAMBDA = 313,
+    TOK_MULTISET_ID = 314,
+    TOK_MULTISET_END = 315,
+    TOK_MULTISET_START = 316,
+    TOK_LSH_EQ = 317,
+    TOK_MAPPING_ID = 318,
+    TOK_MIXED_ID = 319,
+    TOK_MOD_EQ = 320,
+    TOK_MULT_EQ = 321,
+    TOK_NO_MASK = 322,
+    TOK_OBJECT_ID = 323,
+    TOK_OR_EQ = 324,
+    TOK_PRIVATE = 325,
+    TOK_PROGRAM_ID = 326,
+    TOK_PROTECTED = 327,
+    TOK_PREDEF = 328,
+    TOK_PUBLIC = 329,
+    TOK_RSH_EQ = 330,
+    TOK_STATIC = 331,
+    TOK_STRING_ID = 332,
+    TOK_SUB_EQ = 333,
+    TOK_TYPEDEF = 334,
+    TOK_TYPEOF = 335,
+    TOK_VARIANT = 336,
+    TOK_VERSION = 337,
+    TOK_VOID_ID = 338,
+    TOK_WHILE = 339,
+    TOK_XOR_EQ = 340,
+    TOK_OPTIONAL = 341,
+    TOK_SAFE_INDEX = 342,
+    TOK_SAFE_START_INDEX = 343,
+    TOK_BITS = 344
+  };
 #endif
 /* Tokens.  */
 #define TOK_ARROW 258
@@ -217,26 +223,27 @@
 #define TOK_SAFE_START_INDEX 343
 #define TOK_BITS 344
 
-
-
-
+/* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
-typedef union YYSTYPE
-#line 188 "language.yacc"
+typedef union YYSTYPE YYSTYPE;
+union YYSTYPE
 {
+#line 188 "language.yacc" /* yacc.c:1909  */
+
   int number;
   FLOAT_TYPE fnum;
   struct node_s *n;
   char *str;
   void *ptr;
-}
-/* Line 1529 of yacc.c.  */
-#line 235 "y.tab.h"
-	YYSTYPE;
-# define yystype YYSTYPE /* obsolescent; will be withdrawn */
-# define YYSTYPE_IS_DECLARED 1
+
+#line 240 "y.tab.h" /* yacc.c:1909  */
+};
 # define YYSTYPE_IS_TRIVIAL 1
+# define YYSTYPE_IS_DECLARED 1
 #endif
 
 
 
+int yyparse (void);
+
+#endif /* !YY_YY_Y_TAB_H_INCLUDED  */
