@@ -54,7 +54,7 @@
 #define low_yylex low_yylex0
 #define lex_atoi atoi
 #define lex_strtol strtol
-#define lex_strtod my_strtod
+#define lex_strtod mymy_strtod
 #define lex_isidchar isidchar
 
 #else /* SHIFT != 0 */
@@ -144,11 +144,16 @@ static long lex_strtol(char *buf, char **end, int base)
   return ret;
 }
 
-static double lex_strtod(char *buf, char **end)
+static FLOAT_TYPE lex_strtod(char *buf, char **end)
 {
   PCHARP foo;
+#if SIZEOF_FLOAT_TYPE > SIZEOF_DOUBLE
+  FLOAT_TYPE ret;
+  ret=STRTOLD_PCHARP(MKPCHARP(buf,SHIFT),&foo);
+#else
   double ret;
   ret=STRTOD_PCHARP(MKPCHARP(buf,SHIFT),&foo);
+#endif
   if(end) end[0]=(char *)foo.ptr;
   return ret;
 }
@@ -972,7 +977,7 @@ unknown_directive:
     case '5': case '6': case '7': case '8': case '9':
     {
       char *p1, *p2;
-      double f;
+      FLOAT_TYPE f;
       long l;
       struct svalue sval;
 
@@ -1026,10 +1031,10 @@ unknown_directive:
 	  }
 	}
 	free_svalue(&sval);
-	yylval->fnum=(FLOAT_TYPE)f;
+	yylval->fnum=f;
 #if 0
-	fprintf(stderr, "LEX: \"%.8s\" => %f, %f\n",
-		(char *)lex->pos, f, yylval->fnum);
+	fprintf(stderr, "LEX: \"%.8s\" => %"PRINTPIKEFLOAT"f\n",
+		(char *)lex->pos, f);
 #endif /* 0 */
 	lex->pos=p1;
 	if (lex_isidchar (LOOK())) {
@@ -1062,7 +1067,11 @@ unknown_directive:
 
     case '-':
       if(GOBBLE('=')) return TOK_SUB_EQ;
-      if(GOBBLE('>')) return TOK_ARROW;
+      if(GOBBLE('>'))
+	if(GOBBLE('?') ) /* ->? */
+	  return TOK_SAFE_INDEX;
+	else
+	  return TOK_ARROW;
       if(GOBBLE('-')) return TOK_DEC;
       return '-';
   
@@ -1132,24 +1141,13 @@ unknown_directive:
       if(GOBBLE(':'))
         return TOK_LOR;
 
-      if(GOBBLE('-') ) /* safe index: ?->  or ?[] */
+      if(GOBBLE('-') ) /* old safe index: ?->, depreceated in 8.1+ */
       {
         if( GOBBLE( '>' ) ) /* ?-> */
             return TOK_SAFE_INDEX;
         SKIPN(-1); /* Undo GOBBLE('-') above */
       }
 
-      /* Probably wanted:
-
-         ?.   for safe constant index
-         ?[]  for safe [] index
-
-         They however conflict with valid ?: syntaxes.
-      */
-
-      /* if( GOBBLE('.' ) ) */
-      /*   return TOK_SAFE_INDEX; */
-
     case ']':
     case ',':
     case '~':
