@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 4d8746c546cd2bdf6edf91c2046fe14eddef7f17 $
 */
 
 /* MUST BE FIRST */
@@ -61,7 +60,7 @@ PIKE_MODULE_INIT
 {
   int i;
 
-  MEMSET(char_class, CLS_TOKEN, sizeof(char_class));
+  memset(char_class, CLS_TOKEN, sizeof(char_class));
   for(i='\0'; i<=' '; i++)
     char_class[i] = CLS_WSPACE;
   for(i='0'; i<='9'; i++)
@@ -167,25 +166,25 @@ static void f_read( INT32 args )
   int bufsize=CLF_BLOCK_SIZE, bufpos=0;
 #endif
 
-  if(args>2 && sp[-1].type == T_INT) {
+  if(args>2 && TYPEOF(sp[-1]) == T_INT) {
     offs0 = sp[-1].u.integer;
     pop_n_elems(1);
     --args;
   }
   old_sp = sp;
 
-  get_all_args("CommonLog.read", args, "%*%*", &logfun, &file);
-  if(logfun->type != T_FUNCTION)
-    SIMPLE_BAD_ARG_ERROR("CommonLog.read", 1, "function");
+  get_all_args("read", args, "%*%*", &logfun, &file);
+  if(TYPEOF(*logfun) != T_FUNCTION)
+    SIMPLE_BAD_ARG_ERROR("read", 1, "function");
 
-  if(file->type == T_OBJECT)
+  if(TYPEOF(*file) == T_OBJECT)
   {
     f = fd_from_object(file->u.object);
     
     if(f == -1)
-      Pike_error("CommonLog.read: File is not open.\n");
+      Pike_error("File is not open.\n");
     my_fd = 0;
-  } else if(file->type == T_STRING &&
+  } else if(TYPEOF(*file) == T_STRING &&
 	    file->u.string->size_shift == 0) {
 #ifdef PIKE_SECURITY
       if(!CHECK_SECURITY(SECURITY_BIT_SECURITY))
@@ -199,14 +198,14 @@ static void f_read( INT32 args )
 	push_int(00666);
 
 	safe_apply(OBJ2CREDS(CURRENT_CREDS)->user,"valid_open",5);
-	switch(Pike_sp[-1].type)
+	switch(TYPEOF(Pike_sp[-1]))
 	{
 	case PIKE_T_INT:
 	  switch(Pike_sp[-1].u.integer)
 	  {
 	  case 0: /* return 0 */
 	    errno=EPERM;
-	    Pike_error("CommonLog.read(): Failed to open file for reading (errno=%d).\n",
+	    Pike_error("Failed to open file for reading (errno=%d).\n",
 		       errno);
 
 	  case 2: /* ok */
@@ -214,7 +213,7 @@ static void f_read( INT32 args )
 	    break;
 
 	  case 3: /* permission denied */
-	    Pike_error("CommonLog.read: permission denied.\n");
+	    Pike_error("permission denied.\n");
 
 	  default:
 	    Pike_error("Error in user->valid_open, wrong return value.\n");
@@ -222,8 +221,10 @@ static void f_read( INT32 args )
 	  break;
 
 	default:
+#ifdef PIKE_DEBUG
 	  Pike_error("Error in user->valid_open, wrong return type.\n");
-
+#endif
+          /* Fallthrough */
 	case PIKE_T_STRING:
 	  /*	  if(Pike_sp[-1].u.string->shift_size) */
 	  /*	    file=Pike_sp[-1]; */
@@ -241,10 +242,10 @@ static void f_read( INT32 args )
     } while (1);
 
     if(f < 0)
-      Pike_error("CommonLog.read(): Failed to open file for reading (errno=%d).\n",
-	    errno);
+      Pike_error("Failed to open file for reading (errno=%d).\n",
+                 errno);
   } else
-    SIMPLE_BAD_ARG_ERROR("CommonLog.read", 2, "string|Stdio.File");
+    SIMPLE_BAD_ARG_ERROR("read", 2, "string|Stdio.File");
 
 #ifdef HAVE_LSEEK64
   lseek64(f, offs0, SEEK_SET);
