@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 88d4b11589ea9636027e26f88c5e470366c8d347 $
 */
 
 /*
@@ -45,7 +44,9 @@
 
 #if HAVE_DVB > 19
 #  include <linux/dvb/version.h>
-#  include <linux/dvb/sec.h>
+#  ifdef HAVE_LINUX_DVB_SEC_H
+#    include <linux/dvb/sec.h>
+#  endif
 #  include <linux/dvb/frontend.h>
 #  include <linux/dvb/dmx.h>
 #  include <linux/dvb/audio.h>
@@ -76,9 +77,39 @@
 #include "module_support.h"
 #include "builtin_functions.h"
 #include "operators.h"
+#include "pike_types.h"
 
 #include "dvb.h"
 
+#if HAVE_DVB >= 30
+/* Compat with old DVB. */
+/* frontend.h */
+typedef struct dvb_frontend_info FrontendInfo;
+typedef struct dvb_frontend_parameters FrontendParameters;
+typedef struct dvb_frontend_event FrontendEvent;
+#define minFrequency frequency_min
+#define maxFrequency frequency_max
+#define minSymbolRate symbol_rate_min
+#define maxSymbolRate symbol_rate_max
+#define hwType type
+#define Frequency frequency
+#define SymbolRate symbol_rate
+#define FEC_inner fec_inner
+/* audio.h */
+typedef audio_status_t audioStatus_t;
+typedef audio_mixer_t audioMixer_t;
+#define AVSyncState AV_sync_state
+#define muteState mute_state
+#define playState play_state
+#define streamSource stream_source
+#define channelSelect channel_select
+#define bypassMode bypass_mode
+/* dmx.h */
+typedef __u16 dvb_pid_t;
+#define dmxSctFilterParams dmx_sct_filter_params
+#define dmxPesFilterParams dmx_pes_filter_params
+#define pesType pes_type
+#endif
 
 /* WARNING: It is a design limit of DVB-S full cards! */
 #define MAX_PES_FD	8
@@ -193,14 +224,11 @@ dvb_stream_data *sl_getstream(dvb_data *parent, unsigned int pid) {
 
 }
 
-char *mk_devname(int devno, char *basename) {
+static char devname_buf[24];
 
-  char *devname;
-
-  if((devname = malloc(sizeof(basename)+4)) == NULL)
-      return NULL;
-  sprintf(devname, "%s%d", basename, devno); /* FIXME: uncorrect for v2.0+ !! */
-  return devname;
+static INLINE char *mk_devname(int devno, const char *basename) {
+  sprintf(devname_buf, "%s%d", basename, devno); /* FIXME: uncorrect for v2.0+ !! */
+  return devname_buf;
 }
 
 
@@ -232,29 +260,27 @@ char *mk_devname(int devno, char *basename) {
  */
 static void f_create(INT32 args) {
 
-  int fefd;
+  int fefd, devno;
   char *devname;
 
   if(DVB->cardn != -1)
     Pike_error("Create already called!\n");
 
-  DVB->cardn = 0;
+  devno = 0;
   if(args) {
-    if(Pike_sp[-1].type != T_INT)
+    if(TYPEOF(Pike_sp[-1]) != T_INT)
       Pike_error("Invalid argument 1, expected int.\n");
     else
-      DVB->cardn = (u_short)Pike_sp[-1].u.integer;
+      devno = (u_short)Pike_sp[-1].u.integer;
   }
 
-  if((devname = mk_devname(DVB->cardn, FRONTENDDEVICE)) == NULL)
-      Pike_error("Internal error: can't malloc buffer.\n");
+  devname = mk_devname(devno, FRONTENDDEVICE);
   fefd = open (devname, O_RDWR | O_NONBLOCK);
   if (fefd < 0) {
-      DVB->cardn = -1;
-      /* free(devname); */
       Pike_error("Opening frontend '%s' failed.\n", devname);
   }
   DVB->fefd = fefd;
+  DVB->cardn = devno;
 
   /* Make sure this fd gets closed on exec. */
   set_close_on_exec(fefd, 1);
@@ -304,15 +330,18 @@ static void f_fe_status(INT32 args) {
   if(ret < 0)
     push_int(0);
   else {
-    push_text("power"); push_int(!!(status & ~FE_HAS_POWER));
     push_text("signal"); push_int(!!(status & ~FE_HAS_SIGNAL));
-    /*push_text("spectrum_inverse"); push_int(status & ~FE_HAS_SPECTRUM_INV);*/
-    push_text("lock"); push_int(!!(status & ~FE_HAS_LOCK));
     push_text("carrier"); push_int(!!(status & ~FE_HAS_CARRIER));
     push_text("viterbi"); push_int(!!(status & ~FE_HAS_VITERBI));
+    push_text("lock"); push_int(!!(status & ~FE_HAS_LOCK));
     push_text("sync"); push_int(!!(status & ~FE_HAS_SYNC));
+    cnt = 5;
+    /*push_text("spectrum_inverse"); push_int(status & ~FE_HAS_SPECTRUM_INV);*/
+#if HAVE_DVB < 30
+    push_text("power"); push_int(!!(status & ~FE_HAS_POWER));
     push_text("tuner_lock"); push_int(!!(status & ~FE_TUNER_HAS_LOCK));
-    cnt = 7;
+    cnt += 2;
+#endif
     THREADS_ALLOW();
     ret = ioctl(dvb->fefd, FE_READ_BER, &status);
     THREADS_DISALLOW();
@@ -361,25 +390,37 @@ static void f_fe_info(INT32 args) {
     push_int(0);
   else {
     push_text("frequency");
-      push_text("min"); push_int(info.minFrequency);
-      push_text("max"); push_int(info.maxFrequency);
+      push_text("min");
+      push_int(info.minFrequency);
+      push_text("max");
+      push_int(info.maxFrequency);
       f_aggregate_mapping(2 * 2);
     push_text("sr");
-      push_text("min"); push_int(info.minSymbolRate);
-      push_text("max"); push_int(info.maxSymbolRate);
+      push_text("min");
+      push_int(info.minSymbolRate);
+      push_text("max");
+      push_int(info.maxSymbolRate);
       f_aggregate_mapping(2 * 2);
     push_text("hardware");
-      push_text("type"); push_int(info.hwType);
-      push_text("version"); push_int(info.hwVersion);
+      ref_push_string(literal_type_string);
+      push_int(info.hwType);
+#if HAVE_DVB < 30
+      push_text("version");
+      push_int(info.hwVersion);
       f_aggregate_mapping(2 * 2);
+#else
+      /* FIXME: Where is the version in the new DVB API? */
+      f_aggregate_mapping(1 * 2);
+#endif
     f_aggregate_mapping(2 * 3);
   }
 }
 
 /* digital satellite equipment control,
- * specification is available from http://www.eutelsat.com/ 
+ * specification is available from http://www.eutelsat.com/
  */
 static int diseqc (int secfd, int sat_no, int pol, int hi_lo) {
+#if HAVE_DVB < 30
   struct secCmdSequence secseq;
   struct secCommand scmd;
 
@@ -401,7 +442,27 @@ static int diseqc (int secfd, int sat_no, int pol, int hi_lo) {
   secseq.commands = &scmd;
 
   if (ioctl (secfd, SEC_SEND_SEQUENCE, &secseq) == -1)
-    return 0; 
+    return 0;
+#else
+  struct dvb_diseqc_master_cmd cmd;
+  cmd.msg[0] = 0xe0;	/* framing (master, no reply expected, first send) */
+  cmd.msg[1] = 0x10;	/* address (any switch) */
+  cmd.msg[2] = 0x38;	/* command (set port group 0) */
+  /* param: high nibble: reset bits, low nibble set bits,
+   * bits are: option, position, polarizaion, band
+   */
+  cmd.msg[3] = 0xF0 | (((sat_no * 4) & 0x0F)
+                                    | (hi_lo?1:0) | (pol?0:2));
+  cmd.msg_len = 4;
+
+  if (ioctl (secfd, FE_DISEQC_SEND_MASTER_CMD, &cmd) == -1) {
+    /* tone/volt for backwards compatiblity with non-diseqc LNBs */
+    if (ioctl (secfd, FE_SET_VOLTAGE, pol ? SEC_VOLTAGE_13 : SEC_VOLTAGE_18) == -1)
+      return 0;
+    if (ioctl (secfd, FE_SET_TONE, hi_lo ? SEC_TONE_ON : SEC_TONE_OFF) == -1)
+      return 0;
+  }
+#endif
   return 1;
 }
 
@@ -449,15 +510,21 @@ static int do_tune(int fefd, uint ifreq, uint sr)
       return 0;
   }
 
+#if HAVE_DVB < 30
   if (ev.type != FE_COMPLETION_EV) {
       snprintf (DVB->low_errmsg, MAX_ERR_LEN, "tuning failed\n");
       return 0;
   }
+#else
+  if (ev.status != FE_HAS_LOCK) {
+      snprintf (DVB->low_errmsg, MAX_ERR_LEN, "tuning failed\n");
+      return 0;
+  }
+#endif
 
   return 1;
 }
 
-
 /*! @decl int tune(int(0..3) lnb, int freq, int(0..1)|string pol, int sr)
  *!
  *! Tunes to apropriate transponder's parameters.
@@ -495,7 +562,7 @@ static void f_zap(INT32 args) {
   sr = (u_short)Pike_sp[-1].u.integer * 1000;
   Pike_sp--;
 
-  if(Pike_sp[-1].type == T_INT)
+  if(TYPEOF(Pike_sp[-1]) == T_INT)
       pol = (u_short)Pike_sp[-1].u.integer;
   else
       pol = Pike_sp[-1].u.string->str[0] == 'V' ||
@@ -507,13 +574,16 @@ static void f_zap(INT32 args) {
 
   satno = (u_short)Pike_sp[-1].u.integer;
 
-  if((devname = mk_devname(dvb->cardn, SECDEVICE)) == NULL)
-      Pike_error("Internal error: can't malloc buffer.\n");
+#if HAVE_DVB < 30
+  devname = mk_devname(dvb->cardn, SECDEVICE);
   secfd = open (devname, O_RDWR);
   /* free(devname); */
   if (secfd == -1) {
       Pike_error ("opening SEC device failed\n");
   }
+#else
+  secfd = dvb->fefd;
+#endif
   THREADS_ALLOW();
   result = ioctl (dvb->fefd, FE_GET_INFO, &fe_info);
   THREADS_DISALLOW();
@@ -541,7 +611,6 @@ static void f_zap(INT32 args) {
   push_int(result);
 }
 
-
 /*! @decl mapping|int get_pids()
  *!
  *! Returns mapping with info of currently tuned program's pids.
@@ -551,14 +620,13 @@ static void f_zap(INT32 args) {
  */
 static void f_get_pids(INT32 args) {
 
-  dvb_pid_t pids[7];
+  dvb_pid_t pids[5];
   int cnt = 0, dmx, ret;
   char *devname;
 
   pop_n_elems(args);
   if(!sl_count(DVB)) {
-    if((devname = mk_devname(DVB->cardn, DEMUXDEVICE)) == NULL)
-        Pike_error("Internal error: can't malloc buffer.\n");
+    devname = mk_devname(DVB->cardn, DEMUXDEVICE);
     dmx = open (devname, O_RDWR | O_NONBLOCK);
     /* free(devname); */
     if (dmx < 0)
@@ -585,12 +653,7 @@ static void f_get_pids(INT32 args) {
     cnt++;
     push_text("pcr");		push_int( pids[DMX_PES_PCR] & 0x1fff );
     cnt++;
-    push_text("other");		push_int( pids[DMX_PES_OTHER] & 0x1fff );
-    cnt++;
-    if(cnt)
-      f_aggregate_mapping( 2 * cnt );
-    else
-      push_int(0);
+    f_aggregate_mapping( 2 * cnt );
   } else
     push_int(0);
   if(!sl_count(DVB))
@@ -739,7 +802,7 @@ static int read_t(int fd,unsigned char *buffer,int length,int cks)
       return -1;
     }
 
-    if (cks && crc32(buffer+1,n) != 0)
+    if (cks && crc32((char *)buffer+1,n) != 0)
     {
       fprintf(stderr,"crc error\n"); /* FIXME: ??? */
       continue;
@@ -770,10 +833,8 @@ static void f_parse_pat(INT32 args) {
   char *devname;
 
   pop_n_elems(args);
-  if((devname = mk_devname(DVB->cardn, DEMUXDEVICE)) == NULL)
-     Pike_error("Internal error: can't malloc buffer.\n");
+  devname = mk_devname(DVB->cardn, DEMUXDEVICE);
   dmx = open (devname, O_RDWR | O_NONBLOCK);
-  /* free(devname); */
   if (dmx < 0) {
     snprintf (DVB->low_errmsg, MAX_ERR_LEN, "DMX SET SECTION FILTER.\n");
     push_int(0);
@@ -934,10 +995,8 @@ static void f_parse_pmt(INT32 args)
 
   check_all_args("DVB.dvb->analyze_pmt", args, BIT_INT, BIT_INT, 0);
 
-  if((devname = mk_devname(DVB->cardn, DEMUXDEVICE)) == NULL)
-     Pike_error("Internal error: can't malloc buffer.\n");
+  devname = mk_devname(DVB->cardn, DEMUXDEVICE);
   dmx = open (devname, O_RDWR | O_NONBLOCK);
-  /* free(devname); */
   if (dmx < 0) {
     snprintf (DVB->low_errmsg, MAX_ERR_LEN, "DMX SET SECTION FILTER.\n");
     push_int(0);
@@ -1003,20 +1062,20 @@ static void f_parse_pmt(INT32 args)
 
     switch(buffer[index]) {
       case 2:
-        push_text("type"); push_int(DMX_PES_VIDEO); cnt++;
+        ref_push_string(literal_type_string); push_int(DMX_PES_VIDEO); cnt++;
         push_text("desc"); push_text("video"); cnt++;
 	break;
       case 3:
       case 4:
-        push_text("type"); push_int(DMX_PES_AUDIO); cnt++;
+        ref_push_string(literal_type_string); push_int(DMX_PES_AUDIO); cnt++;
         push_text("desc"); push_text("audio"); cnt++;
 	break;
       case 6:
-        push_text("type"); push_int(DMX_PES_TELETEXT); cnt++;
+        ref_push_string(literal_type_string); push_int(DMX_PES_TELETEXT); cnt++;
         push_text("desc"); push_text("teletext"); cnt++;
 	break;
       case 129:
-        push_text("type"); push_int(_DMX_PES_RDS); cnt++;
+        ref_push_string(literal_type_string); push_int(_DMX_PES_RDS); cnt++;
         push_text("desc"); push_text("_rds"); cnt++;
 	break;
       default:
@@ -1035,7 +1094,7 @@ static void f_parse_pmt(INT32 args)
           case 0x0a:
             if (buffer[index]==3 || buffer[index]==4) {
 	      push_text("lang");
-	      push_string(make_shared_binary_string(&buffer[i+2], 3)); cnt++;
+	      push_string(make_shared_binary_string((char *)&buffer[i+2], 3)); cnt++;
             }
             break;
           case 0x09:
@@ -1118,7 +1177,7 @@ static void f_stream_attach(INT32 args) {
   push_int(ptype);
   push_object( clone_object(dvb_stream_program, 4 ) );
 
-  if(Pike_sp[-1].type!=PIKE_T_OBJECT)
+  if(TYPEOF(Pike_sp[-1]) != PIKE_T_OBJECT)
     Pike_error("Failed to create Stream object!\n");
 
 }
@@ -1224,13 +1283,11 @@ static void f_stream_create(INT32 args) {
 
   dvbprog = Pike_sp[-1].u.object;
   Pike_sp--;
-  if( !dvbprog || !(dvbstor = (dvb_data *)get_storage( dvbprog, dvb_program )) )
+  if( !dvbprog || !(dvbstor = get_storage( dvbprog, dvb_program )) )
     Pike_error("This class cannot be instantiated directly\n");
 
-  if((devname = mk_devname(DVB->cardn, DEMUXDEVICE)) == NULL)
-     Pike_error("Internal error: can't malloc buffer.\n");
+  devname = mk_devname(DVB->cardn, DEMUXDEVICE);
   fd = open (devname, O_RDWR /*| O_NONBLOCK*/);
-  /* free(devname); */
   if (fd < 0) {
       Pike_error("Opening DEMUX failed.\n");
   }
@@ -1310,7 +1367,7 @@ static void f_stream_read(INT32 args) {
 
   dvb_stream_data *dvb_stream = DVBStream;
   int all = 1, ret, e, cnt, ix = 0;
-  char buf[MAX_DVB_READ_SIZE], *bufptr;
+  unsigned char buf[MAX_DVB_READ_SIZE], *bufptr;
 
   if(dvb_stream->fd < 0)
     Pike_error("Object destroyed!\n");
@@ -1434,16 +1491,14 @@ static void f_audio_create(INT32 args) {
     Pike_error("Create already called!\n");
 
   if(args) {
-    if(Pike_sp[-1].type != T_INT)
+    if(TYPEOF(Pike_sp[-1]) != T_INT)
       Pike_error("Invalid argument 1, expected int.\n");
     else
       devno = (u_short)Pike_sp[-1].u.integer;
   }
   pop_n_elems(args);
-  if((devname = mk_devname(devno, AUDIODEVICE)) == NULL)
-     Pike_error("Internal error: can't malloc buffer.\n");
+  devname = mk_devname(devno, AUDIODEVICE);
   DVBAudio->fd = open( devname, O_RDWR );
-  /* free(devname); */
   if (DVBAudio->fd < 0) {
       DVB->cardn = -1;
       Pike_error("Opening audio device failed.\n");
@@ -1543,7 +1598,7 @@ static void f_audio_ctrl(INT32 args) {
 
   check_all_args("DVB.dvb->ctrl", args, BIT_INT | BIT_STRING, 0);
 
-  if(Pike_sp[-1].type == T_INT)
+  if(TYPEOF(Pike_sp[-1]) == T_INT)
     cw = (u_short)Pike_sp[-1].u.integer;
   else 
     if(!strcmp(Pike_sp[-1].u.string->str, "play"))
@@ -1610,7 +1665,7 @@ static void f_audio_mixer(INT32 args) {
 /*! @endmodule
  */
 
-static void init_dvb_data(struct object *obj) {
+static void init_dvb_data(struct object *UNUSED(obj)) {
 
   unsigned int i;
 
@@ -1620,7 +1675,7 @@ static void init_dvb_data(struct object *obj) {
 }
 
 static void exit_dvb_stream(struct object *obj);
-static void exit_dvb_data(struct object *obj) {
+static void exit_dvb_data(struct object *UNUSED(obj)) {
 
   dvb_stream_data *s;
 
@@ -1634,17 +1689,17 @@ static void exit_dvb_data(struct object *obj) {
   }
 }
 
-static void init_dvb_audio(struct object *obj) {
+static void init_dvb_audio(struct object *UNUSED(obj)) {
   DVBAudio->fd = -1;
   memset(&DVBAudio->low_errmsg, '\0', sizeof(DVBAudio->low_errmsg));
 }
 
-static void exit_dvb_audio(struct object *obj) {
+static void exit_dvb_audio(struct object *UNUSED(obj)) {
   if(DVBAudio->fd != -1)
     close(DVBAudio->fd);
 }
 
-static void init_dvb_stream(struct object *obj) {
+static void init_dvb_stream(struct object *UNUSED(obj)) {
 
   DVBStream->parent = NULL;
   DVBStream->next = NULL;
@@ -1657,7 +1712,7 @@ static void init_dvb_stream(struct object *obj) {
 }
 
 
-static void exit_dvb_stream(struct object *obj) {
+static void exit_dvb_stream(struct object *UNUSED(obj)) {
 
   struct ECMINFO *e;
   unsigned int i;
@@ -1737,7 +1792,7 @@ PIKE_MODULE_INIT {
     add_function("read", f_stream_read, "function(int|void:string|int)", 0);
     add_function("set_buffer", f_stream_set_buffer, "function(int:int)", 0);
     add_function("info", f_stream_info, "function(int:mapping|int)", 0);
-    add_function("close", f_stream_info, "function(:void)", 0);
+    add_function("close", f_stream_close, "function(:void)", 0);
 
   dvb_stream_program = end_program();
   add_program_constant("Stream", dvb_stream_program, 0);
@@ -1760,6 +1815,10 @@ PIKE_MODULE_INIT {
 
 PIKE_MODULE_EXIT {
 
+  if(dvb_stream_program) {
+    free_program(dvb_stream_program);
+    dvb_stream_program = NULL;
+  }
   if(dvb_program) {
     free_program(dvb_program);
     dvb_program = NULL;
@@ -1780,3 +1839,4 @@ PIKE_MODULE_EXIT {
 }
 
 #endif
+
