@@ -728,6 +728,17 @@ void i_img_bmp__decode(INT32 args,int header_only)
    
    if (bpp!=24 && bpp!=16) /* get palette */
    {
+      switch (bpp)
+      {
+         case 8:
+	 case 4:
+	 case 1:
+	    break;
+	 default:
+            Pike_error("Image.BMP.decode: Unexpected bits per pixel value (%d) in image with palette.\n");
+
+      }
+
       push_text("colortable");
 
       if (windows)
@@ -764,6 +775,10 @@ void i_img_bmp__decode(INT32 args,int header_only)
 
    push_int(xsize);
    push_int(abs(ysize));
+
+   if (!(xsize && ysize))
+      Pike_error("Image.BMP.decode: Cannot decode image with less than two dimensions.\n");
+
    push_object(o=clone_object(image_program,2));
    img=get_storage(o,image_program);
    n++;
