@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 92bbdae28f0f8a4acb741a5b68bed0ce7d18b94f $
 */
 
 /* template for operator layer row function */
@@ -17,21 +16,25 @@ static void LM_FUNC(rgb_group *s,rgb_group *l,rgb_group *d,
    if (alpha==0.0)
    {
 #ifdef LAYER_DUAL
-      MEMCPY(d,s,sizeof(rgb_group)*len);
-      MEMCPY(da,sa,sizeof(rgb_group)*len);
+      memcpy(d,s,sizeof(rgb_group)*len);
+      memcpy(da,sa,sizeof(rgb_group)*len);
 #endif
       return; 
    }
    else if (alpha==1.0)
    {
 #ifdef L_COPY_ALPHA
-      MEMCPY(da,sa,sizeof(rgb_group)*len);
+      memcpy(da,sa,sizeof(rgb_group)*len);
 #define da da da da /* error */
 #endif
       if (!la)  /* no layer alpha => full opaque */
       {
 #ifdef L_MMX_OPER
 #ifdef TRY_USE_MMX
+/* Intentionally left here.
+
+   FIXME: Switch to intrinsics
+*/
 	extern int try_use_mmx;
 	if(try_use_mmx)
 	{
@@ -39,7 +42,7 @@ static void LM_FUNC(rgb_group *s,rgb_group *l,rgb_group *d,
 	  unsigned char *source=(unsigned char *)s;
 	  unsigned char *dest=(unsigned char *)d;
 	  unsigned char *sourcel=(unsigned char *)l;
-	  
+
 	  while (num-->0 && (7&(int)dest))
 	  {
 	    *dest=L_TRUNC(L_OPER(*source,*sourcel));
@@ -47,8 +50,8 @@ static void LM_FUNC(rgb_group *s,rgb_group *l,rgb_group *d,
 	    sourcel++;
 	    dest++;
 	  }
-	  
-	  
+
+
 	  while(num > 16)
 	  {
 	    movq_m2r(*source, mm0);
@@ -154,7 +157,7 @@ static void LM_FUNC(rgb_group *s,rgb_group *l,rgb_group *d,
    {
 #ifdef L_COPY_ALPHA
 #undef da
-      MEMCPY(da,sa,sizeof(rgb_group)*len);
+      memcpy(da,sa,sizeof(rgb_group)*len);
 #define da da da
 #endif
       if (!la)  /* no layer alpha => full opaque */
