@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 64c4b10212961dd68b9f54d8c859b21a09d4dd40 $
 */
 
 /*
@@ -92,14 +91,14 @@ static INLINE int getrgb(struct image *img,
    if (args-args_start<3) return 0;
 
    for (i=0; i<3; i++)
-      if (sp[-args+i+args_start].type!=T_INT)
+      if (TYPEOF(sp[-args+i+args_start]) != T_INT)
          Pike_error("Illegal r,g,b argument to %s\n",name);
    img->rgb.r=(unsigned char)sp[-args+args_start].u.integer;
    img->rgb.g=(unsigned char)sp[1-args+args_start].u.integer;
    img->rgb.b=(unsigned char)sp[2-args+args_start].u.integer;
 
    if (args-args_start>=4) {
-      if (sp[3-args+args_start].type!=T_INT) {
+     if (TYPEOF(sp[3-args+args_start]) != T_INT) {
          Pike_error("Illegal alpha argument to %s\n",name);
       }
 
@@ -112,19 +111,6 @@ static INLINE int getrgb(struct image *img,
 }
 
 
-static INLINE int getrgbl(rgbl_group *rgb,INT32 args_start,INT32 args,char *name)
-{
-   INT32 i;
-   if (args-args_start<3) return 0;
-   for (i=0; i<3; i++)
-      if (sp[-args+i+args_start].type!=T_INT)
-         Pike_error("Illegal r,g,b argument to %s\n",name);
-   rgb->r=sp[-args+args_start].u.integer;
-   rgb->g=sp[1-args+args_start].u.integer;
-   rgb->b=sp[2-args+args_start].u.integer;
-   return 1;
-}
-
 /** end internals **/
 
 
@@ -216,7 +202,7 @@ CHRONO("scale begin");
 			source->img, y, source->xsize);
    }
 
-   dest->img=d=malloc(newx*newy*sizeof(rgb_group)+1);
+   dest->img=d=malloc(newx*newy*sizeof(rgb_group)+RGB_VEC_PAD);
    if (d) 
    {
 
@@ -258,10 +244,10 @@ void img_scale2(struct image *dest, struct image *source)
    if (!newx) newx = 1;
    if (!newy) newy = 1;
 
-   new=xalloc(newx*newy*sizeof(rgb_group)+1);
+   new=xalloc(newx*newy*sizeof(rgb_group)+RGB_VEC_PAD);
 
    THREADS_ALLOW();
-   MEMSET(new,0,newx*newy*sizeof(rgb_group));
+   memset(new,0,newx*newy*sizeof(rgb_group));
 
    dest->img=new;
    dest->xsize=newx;
@@ -300,7 +286,7 @@ void img_scale2(struct image *dest, struct image *source)
        pixel(dest,newx,y).g = (COLORTYPE)
 	 (((INT32) pixel(source,2*newx,2*y+0).g+
 	   (INT32) pixel(source,2*newx,2*y+1).g) >> 1);
-       pixel(dest,newx,y).g = (COLORTYPE)
+       pixel(dest,newx,y).b = (COLORTYPE)
 	 (((INT32) pixel(source,2*newx,2*y+0).b+
 	   (INT32) pixel(source,2*newx,2*y+1).b) >> 1);
      }
@@ -365,13 +351,13 @@ void image_scale(INT32 args)
    o=clone_object(image_program,0);
    newimg=(struct image*)(o->storage);
 
-   if (args==1 && sp[-args].type==T_INT)
+   if (args==1 && TYPEOF(sp[-args]) == T_INT)
    {
       free_object(o);
       image_bitscale( args );
       return;
    }
-   else if (args==1 && sp[-args].type==T_FLOAT) 
+   else if (args==1 && TYPEOF(sp[-args]) == T_FLOAT)
    {
       if (sp[-args].u.float_number == 0.5)
 	 img_scale2(newimg,THIS);
@@ -389,8 +375,8 @@ void image_scale(INT32 args)
       }
    }
    else if (args>=2 &&
-	    sp[-args].type==T_INT && sp[-args].u.integer==0 &&
-	    sp[1-args].type==T_INT)
+	    TYPEOF(sp[-args]) == T_INT && sp[-args].u.integer==0 &&
+	    TYPEOF(sp[1-args]) == T_INT)
    {
       factor=((float)sp[1-args].u.integer)/THIS->ysize;
       img_scale(newimg,THIS,
@@ -398,8 +384,8 @@ void image_scale(INT32 args)
 		sp[1-args].u.integer);
    }
    else if (args>=2 &&
-	    sp[1-args].type==T_INT && sp[1-args].u.integer==0 &&
-	    sp[-args].type==T_INT)
+	    TYPEOF(sp[1-args]) == T_INT && sp[1-args].u.integer==0 &&
+	    TYPEOF(sp[-args]) == T_INT)
    {
       factor=((float)sp[-args].u.integer)/THIS->xsize;
       img_scale(newimg,THIS,
@@ -407,22 +393,22 @@ void image_scale(INT32 args)
 		(INT32)(THIS->ysize*factor));
    }
    else if (args>=2 &&
-	    sp[-args].type==T_FLOAT &&
-	    sp[1-args].type==T_FLOAT)
+	    TYPEOF(sp[-args]) == T_FLOAT &&
+	    TYPEOF(sp[1-args]) == T_FLOAT)
       img_scale(newimg, THIS,
 		DOUBLE_TO_INT(THIS->xsize*sp[-args].u.float_number),
 		DOUBLE_TO_INT(THIS->ysize*sp[1-args].u.float_number));
    else if (args>=2 &&
-	    sp[-args].type==T_INT &&
-	    sp[1-args].type==T_INT)
+	    TYPEOF(sp[-args]) == T_INT &&
+	    TYPEOF(sp[1-args]) == T_INT)
       img_scale(newimg,THIS,
 		sp[-args].u.integer,
 		sp[1-args].u.integer);
    else
    {
       free_object(o);
-      bad_arg_error("image->scale",sp-args,args,0,"",sp-args,
-		"Bad arguments to image->scale()\n");
+      bad_arg_error("scale",sp-args,args,0,"",sp-args,
+                    "Bad arguments to scale.\n");
    }
    pop_n_elems(args);
    push_object(o);
@@ -460,11 +446,11 @@ void image_ccw(INT32 args)
    o=clone_object(image_program,0);
    img=(struct image*)o->storage;
    *img=*THIS;
-   if (!(img->img=malloc(sizeof(rgb_group)*THIS->xsize*THIS->ysize+1)))
+   if (!(img->img=malloc(sizeof(rgb_group)*THIS->xsize*THIS->ysize+RGB_VEC_PAD)))
    {
       free_object(o);
       SIMPLE_OUT_OF_MEMORY_ERROR("ccw",
-				 sizeof(rgb_group)*THIS->xsize*THIS->ysize+1);
+				 sizeof(rgb_group)*THIS->xsize*THIS->ysize+RGB_VEC_PAD);
    }
    img->xsize=THIS->ysize;
    img->ysize=THIS->xsize;
@@ -493,7 +479,7 @@ static void img_cw(struct image *is,struct image *id)
 
    if (id->img) free(id->img);
    *id=*is;
-   if (!(id->img=malloc(sizeof(rgb_group)*is->xsize*is->ysize+1)))
+   if (!(id->img=malloc(sizeof(rgb_group)*is->xsize*is->ysize+RGB_VEC_PAD)))
       resource_error(NULL,0,0,"memory",0,"Out of memory.\n");
 
    id->xsize=is->ysize;
@@ -519,7 +505,7 @@ void img_ccw(struct image *is,struct image *id)
 
    if (id->img) free(id->img);
    *id=*is;
-   if (!(id->img=malloc(sizeof(rgb_group)*is->xsize*is->ysize+1)))
+   if (!(id->img=malloc(sizeof(rgb_group)*is->xsize*is->ysize+RGB_VEC_PAD)))
       resource_error(NULL,0,0,"memory",0,"Out of memory.\n");
 
    id->xsize=is->ysize;
@@ -569,11 +555,11 @@ void image_cw(INT32 args)
    o=clone_object(image_program,0);
    img=(struct image*)o->storage;
    *img=*THIS;
-   if (!(img->img=malloc(sizeof(rgb_group)*THIS->xsize*THIS->ysize+1)))
+   if (!(img->img=malloc(sizeof(rgb_group)*THIS->xsize*THIS->ysize+RGB_VEC_PAD)))
    {
       free_object(o);
       SIMPLE_OUT_OF_MEMORY_ERROR("cw",
-				 sizeof(rgb_group)*THIS->xsize*THIS->ysize+1);
+				 sizeof(rgb_group)*THIS->xsize*THIS->ysize+RGB_VEC_PAD);
    }
    ys=img->xsize=THIS->ysize;
    i=xs=img->ysize=THIS->xsize;
@@ -623,11 +609,11 @@ void image_mirrorx(INT32 args)
    o=clone_object(image_program,0);
    img=(struct image*)o->storage;
    *img=*THIS;
-   if (!(img->img=malloc(sizeof(rgb_group)*THIS->xsize*THIS->ysize+1)))
+   if (!(img->img=malloc(sizeof(rgb_group)*THIS->xsize*THIS->ysize+RGB_VEC_PAD)))
    {
       free_object(o);
       SIMPLE_OUT_OF_MEMORY_ERROR("mirrorx",
-				 sizeof(rgb_group)*THIS->xsize*THIS->ysize+1);
+				 sizeof(rgb_group)*THIS->xsize*THIS->ysize+RGB_VEC_PAD);
    }
 
    i=THIS->ysize;
@@ -674,11 +660,11 @@ void image_mirrory(INT32 args)
    o=clone_object(image_program,0);
    img=(struct image*)o->storage;
    *img=*THIS;
-   if (!(img->img=malloc(sizeof(rgb_group)*THIS->xsize*THIS->ysize+1)))
+   if (!(img->img=malloc(sizeof(rgb_group)*THIS->xsize*THIS->ysize+RGB_VEC_PAD)))
    {
       free_object(o);
       SIMPLE_OUT_OF_MEMORY_ERROR("mirrory",
-				 sizeof(rgb_group)*THIS->xsize*THIS->ysize+1);
+				 sizeof(rgb_group)*THIS->xsize*THIS->ysize+RGB_VEC_PAD);
    }
 
    i=THIS->ysize;
@@ -720,7 +706,7 @@ static void img_skewx(struct image *src,
    len=src->xsize;
 
    if (!src->xsize) dest->xsize=0;
-   d=dest->img=malloc(sizeof(rgb_group)*dest->xsize*dest->ysize+1);
+   d=dest->img=malloc(sizeof(rgb_group)*dest->xsize*dest->ysize+RGB_VEC_PAD);
    if (!d) return;
    s=src->img;
 
@@ -808,7 +794,7 @@ static void img_skewy(struct image *src,
    len=src->ysize;
 
    if (!src->ysize) dest->ysize=0;
-   d=dest->img=malloc(sizeof(rgb_group)*dest->ysize*dest->xsize+1);
+   d=dest->img=malloc(sizeof(rgb_group)*dest->ysize*dest->xsize+RGB_VEC_PAD);
    if (!d) return;
    s=src->img;
 
@@ -919,14 +905,14 @@ void image_skewx(INT32 args)
    struct object *o;
 
    if (args<1)
-      SIMPLE_TOO_FEW_ARGS_ERROR("image->skewx",1);
-   else if (sp[-args].type==T_FLOAT)
+      SIMPLE_TOO_FEW_ARGS_ERROR("skewx",1);
+   else if (TYPEOF(sp[-args]) == T_FLOAT)
       diff = THIS->ysize*sp[-args].u.float_number;
-   else if (sp[-args].type==T_INT)
+   else if (TYPEOF(sp[-args])== T_INT)
       diff = (double)sp[-args].u.integer;
    else
-      bad_arg_error("image->skewx",sp-args,args,0,"",sp-args,
-		"Bad arguments to image->skewx()\n");
+      bad_arg_error("skewx",sp-args,args,0,"",sp-args,
+                    "Bad arguments to skewx.\n");
 
    if (!THIS->img) Pike_error("Called Image.Image object is not initialized\n");;
 
@@ -983,14 +969,14 @@ void image_skewy(INT32 args)
    struct object *o;
 
    if (args<1)
-      SIMPLE_TOO_FEW_ARGS_ERROR("image->skewy",1);
-   else if (sp[-args].type==T_FLOAT)
+      SIMPLE_TOO_FEW_ARGS_ERROR("skewy",1);
+   else if (TYPEOF(sp[-args]) == T_FLOAT)
       diff = THIS->xsize*sp[-args].u.float_number;
-   else if (sp[-args].type==T_INT)
+   else if (TYPEOF(sp[-args]) == T_INT)
       diff = (double)sp[-args].u.integer;
    else
-      bad_arg_error("image->skewx",sp-args,args,0,"",sp-args,
-		"Bad arguments to image->skewx()\n");
+      bad_arg_error("skewy",sp-args,args,0,"",sp-args,
+                    "Bad arguments to skewy.\n");
 
    if (!THIS->img) Pike_error("Called Image.Image object is not initialized\n");;
 
@@ -1011,20 +997,20 @@ void image_skewx_expand(INT32 args)
    struct object *o;
 
    if (args<1)
-      SIMPLE_TOO_FEW_ARGS_ERROR("image->skewx",1);
-   else if (sp[-args].type==T_FLOAT)
+      SIMPLE_TOO_FEW_ARGS_ERROR("skewx_expand",1);
+   else if (TYPEOF(sp[-args]) == T_FLOAT)
       diff = THIS->ysize*sp[-args].u.float_number;
-   else if (sp[-args].type==T_INT)
+   else if (TYPEOF(sp[-args]) == T_INT)
       diff = (double)sp[-args].u.integer;
    else
-      bad_arg_error("image->skewx",sp-args,args,0,"",sp-args,
-		"Bad arguments to image->skewx()\n");
+      bad_arg_error("skewx_expand",sp-args,args,0,"",sp-args,
+                    "Bad arguments to skewx_expand.\n");
 
    if (!THIS->img) Pike_error("Called Image.Image object is not initialized\n");;
 
    o=clone_object(image_program,0);
 
-   if (!getrgb((struct image*)(o->storage),1,args,"image->skewx()"))
+   if (!getrgb((struct image*)(o->storage),1,args,"image->skewx_expand()"))
       ((struct image*)(o->storage))->rgb=THIS->rgb;
 
    img_skewx(THIS,(struct image*)(o->storage),diff,1);
@@ -1039,20 +1025,20 @@ void image_skewy_expand(INT32 args)
    struct object *o;
 
    if (args<1)
-      SIMPLE_TOO_FEW_ARGS_ERROR("image->skewy",1);
-   else if (sp[-args].type==T_FLOAT)
+      SIMPLE_TOO_FEW_ARGS_ERROR("skewy_expand",1);
+   else if (TYPEOF(sp[-args]) == T_FLOAT)
       diff = THIS->xsize*sp[-args].u.float_number;
-   else if (sp[-args].type==T_INT)
+   else if (TYPEOF(sp[-args]) == T_INT)
       diff = (double)sp[-args].u.integer;
    else
-      bad_arg_error("image->skewx",sp-args,args,0,"",sp-args,
-		"Bad arguments to image->skewx()\n");
+      bad_arg_error("skewx_expand",sp-args,args,0,"",sp-args,
+                    "Bad arguments to skewy_expand.\n");
 
    if (!THIS->img) Pike_error("Called Image.Image object is not initialized\n");;
 
    o=clone_object(image_program,0);
 
-   if (!getrgb((struct image*)(o->storage),1,args,"image->skewy()"))
+   if (!getrgb((struct image*)(o->storage),1,args,"image->skewy_expand()"))
       ((struct image*)(o->storage))->rgb=THIS->rgb;
 
    img_skewy(THIS,(struct image*)(o->storage),diff,1);
@@ -1070,14 +1056,14 @@ void img_rotate(INT32 args,int xpn)
    struct image *dest,d0,dest2;
 
    if (args<1)
-      SIMPLE_TOO_FEW_ARGS_ERROR("image->rotate",1);
-   else if (sp[-args].type==T_FLOAT)
+      SIMPLE_TOO_FEW_ARGS_ERROR("rotate",1);
+   else if (TYPEOF(sp[-args]) == T_FLOAT)
       angle = sp[-args].u.float_number;
-   else if (sp[-args].type==T_INT)
+   else if (TYPEOF(sp[-args]) == T_INT)
       angle = (double)sp[-args].u.integer;
    else
-      bad_arg_error("image->rotate",sp-args,args,0,"",sp-args,
-		"Bad arguments to image->rotate()\n");
+      bad_arg_error("rotate",sp-args,args,0,"",sp-args,
+                    "Bad arguments to rotate.\n");
 
    if (!THIS->img) 
       Pike_error("Called Image.Image object is not initialized\n");;
@@ -1177,15 +1163,15 @@ void img_translate(INT32 args,int expand)
    if (args<2)
      Pike_error("illegal number of arguments to image->translate()\n");
 
-   if (sp[-args].type==T_FLOAT) xt=sp[-args].u.float_number;
-   else if (sp[-args].type==T_INT) xt=sp[-args].u.integer;
-   else bad_arg_error("image->translate",sp-args,args,1,"",sp+1-1-args,
-		"Bad argument 1 to image->translate()\n");
+   if (TYPEOF(sp[-args]) == T_FLOAT) xt=sp[-args].u.float_number;
+   else if (TYPEOF(sp[-args]) == T_INT) xt=sp[-args].u.integer;
+   else bad_arg_error("translate",sp-args,args,1,"",sp+1-1-args,
+                      "Bad argument 1 to translate.\n");
 
-   if (sp[1-args].type==T_FLOAT) yt=sp[1-args].u.float_number;
-   else if (sp[1-args].type==T_INT) yt=sp[1-args].u.integer;
-   else bad_arg_error("image->translate",sp-args,args,2,"",sp+2-1-args,
-		"Bad argument 2 to image->translate()\n");
+   if (TYPEOF(sp[1-args]) == T_FLOAT) yt=sp[1-args].u.float_number;
+   else if (TYPEOF(sp[1-args]) == T_INT) yt=sp[1-args].u.integer;
+   else bad_arg_error("translate",sp-args,args,2,"",sp+2-1-args,
+                      "Bad argument 2 to translate.\n");
 
    getrgb(THIS,2,args,"image->translate()\n");
 
@@ -1198,16 +1184,16 @@ void img_translate(INT32 args,int expand)
    img->xsize=THIS->xsize+(xt!=0);
    img->ysize=THIS->ysize+(xt!=0);
 
-   if (!(img->img=malloc(sizeof(rgb_group)*img->xsize*img->ysize+1)))
+   if (!(img->img=malloc(sizeof(rgb_group)*img->xsize*img->ysize+RGB_VEC_PAD)))
    {
       free_object(o);
       SIMPLE_OUT_OF_MEMORY_ERROR("translate",
-				 sizeof(rgb_group)*img->xsize*img->ysize+1);
+				 sizeof(rgb_group)*img->xsize*img->ysize+RGB_VEC_PAD);
    }
 
    if (!xt)
    {
-      MEMCPY(img->img,THIS->img,sizeof(rgb_group)*THIS->xsize*THIS->ysize);
+      memcpy(img->img,THIS->img,sizeof(rgb_group)*THIS->xsize*THIS->ysize);
    }
    else
    {
