@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 57351eb041c52b5344918ea17d6b2d9d56896a10 $
 */
 
 #include "global.h"
@@ -305,7 +304,7 @@ static void add_vertices(struct line_list **first,
 
 static void sub_vertices(struct line_list **first,
 			 struct vertex *below,
-			 double yp)
+			 double UNUSED(yp))
 {
    struct line_list **ins,*c;
 
@@ -755,7 +754,7 @@ static INLINE struct vertex *polyfill_add(struct vertex **top,
 #endif
    }
 
-#define POINT(A,N) (((A)->item[N].type==T_FLOAT)?((A)->item[N].u.float_number):((FLOAT_TYPE)((A)->item[N].u.integer)))
+#define POINT(A,N) ((TYPEOF((A)->item[N]) == T_FLOAT)?((A)->item[N].u.float_number):((FLOAT_TYPE)((A)->item[N].u.integer)))
 
    last = first = vertex_new(DO_NOT_WARN(POINT(a,0)),
 			     DO_NOT_WARN(POINT(a,1)),
@@ -815,7 +814,7 @@ void image_polyfill(INT32 args)
    {
       struct vertex *v_tmp;
 
-      if (sp[-1].type!=T_ARRAY)
+      if (TYPEOF(sp[-1]) != T_ARRAY)
       {
 	 polyfill_free(v);
 	 SIMPLE_BAD_ARG_ERROR("Image.Image->polyfill", args,
