@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 990aa05dd09302e55322a00deed712e2e2e29fd9 $
 */
 
 #include "global.h"
@@ -41,6 +40,8 @@
  */
 
 /*! @class PDFgen
+ *! Interface to the pdflib pdf generator.
+ *! For more information see http://www.pdflib.com
  */
 
 /*** PDF callabcks ******************************************************/
@@ -77,6 +78,10 @@ static void exit_pdf(struct object *o)
    }
 }
 
+/*! @decl void create();
+ *
+ * Create a new PDF generation context.
+ */
 static void pdf_create(INT32 args)
 {
    PDF *pdf;
@@ -109,7 +114,7 @@ static void pdf_open_file(INT32 args)
    char *s;
    int n;
    if (args<1) SIMPLE_TOO_FEW_ARGS_ERROR("open_file",1);
-   if (sp[-args].type!=T_STRING || sp[-args].u.string->size_shift)
+   if (TYPEOF(sp[-args]) != T_STRING || sp[-args].u.string->size_shift)
       SIMPLE_BAD_ARG_ERROR("open_file",1,"8 bit string");
    s=sp[-args].u.string->str;
    if (!this->pdf) Pike_error("PDF not initiated\n");
