@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 7f5f55e074aee5e39e26def79bbb342c4fd3b521 $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -51,6 +50,9 @@ static struct program *std_16bite_program = NULL;
 
 static size_t rfc_charset_name_offs = 0;
 
+static struct array *double_custom_chars = NULL;
+static struct array *double_combiner_chars = NULL;
+
 struct std_cs_stor { 
   struct string_builder strbuild;
   struct pike_string *retain, *replace;
@@ -113,13 +115,13 @@ static void f_create(INT32 args)
   check_all_args("create()", args, BIT_STRING|BIT_VOID|BIT_INT,
 		 BIT_FUNCTION|BIT_VOID|BIT_INT, 0);
 
-  if(args>0 && sp[-args].type == T_STRING) {
+  if(args>0 && TYPEOF(sp[-args]) == T_STRING) {
     if(s->replace != NULL)
       free_string(s->replace);
     add_ref(s->replace = sp[-args].u.string);
   }
 
-  if(args>1 && sp[1-args].type == T_FUNCTION)
+  if(args>1 && TYPEOF(sp[1-args]) == T_FUNCTION)
     assign_svalue(&s->repcb, &sp[1-args]);
 
   pop_n_elems(args);
@@ -143,7 +145,7 @@ static int call_repcb(struct svalue *repcb, p_wchar2 ch)
 {
   push_string(make_shared_binary_string2(&ch, 1));
   apply_svalue(repcb, 1);
-  if(sp[-1].type == T_STRING)
+  if(TYPEOF(sp[-1]) == T_STRING)
     return 1;
   pop_stack();
   return 0;
@@ -163,11 +165,11 @@ static void DECLSPEC(noreturn) transcode_error_va (
   struct svalue *err_prog;
 
   if (encode) {
-    if (encode_err_prog.type == T_INT) {
-      push_text ("Locale.Charset.EncodeError");
+    if (TYPEOF(encode_err_prog) == T_INT) {
+      push_text ("Charset.EncodeError");
       SAFE_APPLY_MASTER ("resolv", 1);
-      if (sp[-1].type != T_PROGRAM && sp[-1].type != T_FUNCTION)
-	Pike_error ("Failed to resolve Locale.Charset.EncodeError "
+      if (TYPEOF(sp[-1]) != T_PROGRAM && TYPEOF(sp[-1]) != T_FUNCTION)
+	Pike_error ("Failed to resolve Charset.EncodeError "
 		    "to a program - unable to throw an encode error.\n");
       move_svalue (&encode_err_prog, --sp);
     }
@@ -175,11 +177,11 @@ static void DECLSPEC(noreturn) transcode_error_va (
   }
 
   else {
-    if (decode_err_prog.type == T_INT) {
-      push_text ("Locale.Charset.DecodeError");
+    if (TYPEOF(decode_err_prog) == T_INT) {
+      push_text ("Charset.DecodeError");
       SAFE_APPLY_MASTER ("resolv", 1);
-      if (sp[-1].type != T_PROGRAM && sp[-1].type != T_FUNCTION)
-	Pike_error ("Failed to resolve Locale.Charset.DecodeError "
+      if (TYPEOF(sp[-1]) != T_PROGRAM && TYPEOF(sp[-1]) != T_FUNCTION)
+	Pike_error ("Failed to resolve Charset.DecodeError "
 		    "to a program - unable to throw an decode error.\n");
       move_svalue (&decode_err_prog, --sp);
     }
@@ -218,9 +220,9 @@ void DECLSPEC(noreturn) transcoder_error (
   struct svalue charset_str, charset;
   va_list args;
   va_start (args, reason);
-  charset_str.subtype = 0;
+  SET_SVAL_SUBTYPE(charset_str, 0);
   MAKE_CONST_STRING (charset_str.u.string, "charset");
-  charset_str.type = T_STRING;
+  SET_SVAL_TYPE(charset_str, T_STRING);
   object_index_no_free (&charset, fp->current_object, 0, &charset_str);
   transcode_error_va (str, pos, charset.u.string, encode, reason, args);
   va_end (args);
@@ -233,10 +235,10 @@ void DECLSPEC(noreturn) transcoder_error (
     } else if(rep != NULL)						\
       func(ctx, sb, rep, NULL, NULL);					\
     else								\
-      transcoder_error (str, pos, 1, "Unsupported character.\n");	\
+      transcoder_error (str, pos, 1, "Unsupported character %d.\n",ch);	\
   } while (0)
 
-#define MKREPCB(c) ((c).type == T_FUNCTION? &(c):NULL)
+#define MKREPCB(c) (TYPEOF(c) == T_FUNCTION? &(c):NULL)
 
 static void f_drain(INT32 args)
 {
@@ -251,6 +253,7 @@ static void f_drain_rfc1345(INT32 args)
 {
   struct std_cs_stor *s = (struct std_cs_stor *)fp->current_storage;
   UNICHAR trailer = 0;
+  int double_combiners = 0;
 
   if (s->strbuild.s->size_shift) {
     ptrdiff_t i, len = s->strbuild.s->len;
@@ -260,17 +263,45 @@ static void f_drain_rfc1345(INT32 args)
       {
 	p_wchar1 *s1 = STR1(s->strbuild.s);
 	for (i=0; i < len; i++) {
-	  if ((s1[i] & 0xff00) == 0xe300) {
-	    /* Non-spacing character ==> combiner  */
-	    trailer = s1[i++];
-	    if (i < len) {
-	      s1[i-1] = s1[i];
-	      s1[i] = trailer & 0x0fff;
-	      trailer = 0;
-	    } else {
-	      s->strbuild.s->len--;
-	      break;
-	    }
+	  if ((s1[i] & 0xf000) == 0xe000) {
+	    if ((s1[i] & 0xff00) == 0xe300) {
+	      /* Non-spacing character ==> combiner  */
+	      trailer = s1[i++];
+	      if (i < len) {
+		s1[i-1] = s1[i];
+		s1[i] = trailer & 0x0fff;
+		trailer = 0;
+	      } else {
+		s->strbuild.s->len--;
+		break;
+	      }
+	    } else if ((s1[i] & 0xff00) == 0xee00) {
+	      /* Non-spacing character ==> combiner  */
+	      trailer = s1[i++];
+	      if (i < len) {
+		s1[i-1] = s1[i];
+		s1[i] = 0xf000 | (trailer & 0x0fff);
+		trailer = 0;
+	      } else {
+		s->strbuild.s->len--;
+		break;
+	      }
+	    } else if ((s1[i] & 0xff00) == 0xe100) {
+	      /* Non-spacing character ==> double combiner
+	       *
+	       * Reorder here, and expand later.
+	       */
+	      trailer = s1[i++];
+	      if (i < len) {
+		s1[i-1] = s1[i];
+		s1[i] = trailer;
+		trailer = 0;
+		double_combiners = 1;
+	      } else {
+		s->strbuild.s->len--;
+		break;
+	      }
+	    } 
 	  }
 	}
       }
@@ -279,21 +310,51 @@ static void f_drain_rfc1345(INT32 args)
       {
 	p_wchar2 *s2 = STR2(s->strbuild.s);
 	for (i=0; i < len; i++) {
-	  if ((s2[i] & 0xff00) == 0xe300) {
-	    /* Non-spacing character ==> combiner */
-	    trailer = s2[i++];
-	    if (i < len) {
-	      s2[i-1] = s2[i];
-	      s2[i] = trailer & 0x0fff;
-	      trailer = 0;
-	    } else {
-	      s->strbuild.s->len--;
-	      break;
+	  if ((s2[i] & 0xf000) == 0xe000) {
+	    if ((s2[i] & 0xff00) == 0xe300) {
+	      /* Non-spacing character ==> combiner */
+	      trailer = s2[i++];
+	      if (i < len) {
+		s2[i-1] = s2[i];
+		s2[i] = trailer & 0x0fff;
+		trailer = 0;
+	      } else {
+		s->strbuild.s->len--;
+		break;
+	      }
+	    } else if ((s2[i] & 0xff00) == 0xee00) {
+	      /* Non-spacing character ==> combiner  */
+	      trailer = s2[i++];
+	      if (i < len) {
+		s2[i-1] = s2[i];
+		s2[i] = 0xf000 | (trailer & 0x0fff);
+		trailer = 0;
+	      } else {
+		s->strbuild.s->len--;
+		break;
+	      }
+	    } else if ((s2[i] & 0xff00) == 0xe100) {
+	      /* Non-spacing character ==> double combiner
+	       *
+	       * Reorder here, and expand later.
+	       */
+	      trailer = s2[i++];
+	      if (i < len) {
+		s2[i-1] = s2[i];
+		s2[i] = trailer;
+		trailer = 0;
+		double_combiners = 1;
+	      } else {
+		s->strbuild.s->len--;
+		break;
+	      }
 	    }
 	  }
 	}
       }
       break;
+    case 0: /* Dummy case to silence clang warning */
+      break;
     }
   }
   f_drain(args);
@@ -303,6 +364,12 @@ static void f_drain_rfc1345(INT32 args)
      */
     string_builder_putchar(&s->strbuild, trailer);
   }
+  if (double_combiners) {
+    /* There were non-spacing double modifiers used. */
+    ref_push_array(double_custom_chars);
+    ref_push_array(double_combiner_chars);
+    f_replace(3);
+  }
 }
 
 static void f_clear(INT32 args)
@@ -321,7 +388,7 @@ static void f_clear(INT32 args)
   push_object(this_object());
 }
 
-static void init_stor(struct object *o)
+static void init_stor(struct object *UNUSED(o))
 {
   struct std_cs_stor *s = (struct std_cs_stor *)fp->current_storage;
 
@@ -331,7 +398,7 @@ static void init_stor(struct object *o)
   init_string_builder(&s->strbuild,0);
 }
 
-static void exit_stor(struct object *o)
+static void exit_stor(struct object *UNUSED(o))
 {
   struct std_cs_stor *s = (struct std_cs_stor *)fp->current_storage;
 
@@ -355,7 +422,7 @@ static void f_std_feed(INT32 args, ptrdiff_t (*func)(struct pike_string *,
   struct pike_string *str;
   ptrdiff_t l;
 
-  get_all_args("feed()", args, "%W", &str);
+  get_all_args("feed", args, "%W", &str);
 
   if(str->size_shift>0)
     Pike_error("Can't feed on wide strings!\n");
@@ -691,7 +758,7 @@ static void f_clear_utf7(INT32 args)
   u7->datbit = 0;
 }
 
-static void utf7_init_stor(struct object *o)
+static void utf7_init_stor(struct object *UNUSED(o))
 {
   struct utf7_stor *u7 =
     (struct utf7_stor *)(fp->current_storage+utf7_stor_offs);
@@ -864,7 +931,7 @@ static void f_create_multichar(INT32 args)
   const struct multichar_def *def = multichar_map;
   struct multichar_stor *s = (struct multichar_stor *)(fp->current_storage + multichar_stor_offs);
 
-  get_all_args("create()", args, "%s", &name);
+  get_all_args("create", args, "%s", &name);
 
   while(1) {
     if(def->name == 0)
@@ -1112,7 +1179,7 @@ static void f_feed_gb18030e(INT32 args)
   struct pike_string *str;
   struct std_cs_stor *cs = (struct std_cs_stor *)fp->current_storage;
 
-  get_all_args("feed()", args, "%W", &str);
+  get_all_args("feed", args, "%W", &str);
 
   feed_gb18030e(cs, &cs->strbuild, str, cs->replace, MKREPCB(cs->repcb));
 
@@ -1193,7 +1260,7 @@ static void f_feed_gbke(INT32 args)
   struct pike_string *str;
   struct std_cs_stor *cs = (struct std_cs_stor *)fp->current_storage;
 
-  get_all_args("feed()", args, "%W", &str);
+  get_all_args("feed", args, "%W", &str);
 
   feed_gbke(cs, &cs->strbuild, str, cs->replace, MKREPCB(cs->repcb));
 
@@ -1210,8 +1277,7 @@ static void f_create_sjise(INT32 args)
   s->lo = 0x5c;
   s->hi = 0xfffd;
 
-  memset((s->revtab = (p_wchar1 *)xalloc((s->hi-s->lo)*sizeof(p_wchar1))), 0,
-	 (s->hi-s->lo)*sizeof(p_wchar1));
+  s->revtab = xcalloc(s->hi-s->lo, sizeof(p_wchar1));
 
   for(z=0, i=33; i<=126; i++, z+=94)
     for(j=33; j<=126; j++) {
@@ -1279,8 +1345,7 @@ static void f_create_euce(INT32 args)
   s->lo = 128;
   s->hi = 128;
 
-  memset((s->revtab = (p_wchar1 *)xalloc((65536-s->lo)*sizeof(p_wchar1))), 0,
-	 (65536-s->lo)*sizeof(p_wchar1));
+  s->revtab = xcalloc(65536-s->lo, sizeof(p_wchar1));
 
   for(z=0, i=33; i<=126; i++, z+=94)
     for(j=33; j<=126; j++) {
@@ -1337,8 +1402,7 @@ static struct std8e_stor *push_std_8bite(int args, int allargs, int lo, int hi)
   pop_n_elems(allargs);
   push_object(o);
   s8 = (struct std8e_stor *)(sp[-1].u.object->storage+std8e_stor_offs);
-  memset((s8->revtab = (p_wchar0 *)xalloc((hi-lo)*sizeof(p_wchar0))), 0,
-	 (hi-lo)*sizeof(p_wchar0));
+  s8->revtab = xcalloc(hi-lo, sizeof(p_wchar0));
   s8->lo = lo;
   s8->hi = hi;
   s8->lowtrans = 0;
@@ -1356,8 +1420,7 @@ static struct std16e_stor *push_std_16bite(int args, int allargs, int lo, int hi
   pop_n_elems(allargs);
   push_object(o);
   s16 = (struct std16e_stor *)(sp[-1].u.object->storage+std16e_stor_offs);
-  memset((s16->revtab = (p_wchar1 *)xalloc((hi-lo)*sizeof(p_wchar1))), 0,
-	 (hi-lo)*sizeof(p_wchar1));
+  s16->revtab = xcalloc(hi-lo, sizeof(p_wchar1));
   s16->lo = lo;
   s16->hi = hi;
   s16->lowtrans = 0;
@@ -1384,7 +1447,7 @@ static void f_rfc1345(INT32 args)
     if((c = strcmp((char *)STR0(str), charset_map[mid].name))==0) {
       struct program *p = NULL;
 
-      if(args>1 && sp[1-args].type == T_INT && sp[1-args].u.integer != 0) {
+      if(args>1 && TYPEOF(sp[1-args]) == T_INT && sp[1-args].u.integer != 0) {
 	unsigned lowtrans = 0;
 	int i, j, lo2=0, hi2=0, z, c;
 
@@ -1462,7 +1525,7 @@ static void f_rfc1345(INT32 args)
   if(str->size_shift==0 &&
      (tabl = misc_charset_lookup((char *)STR0(str), &lo, &hi))!=NULL) {
 
-    if(args>1 && sp[1-args].type == T_INT && sp[1-args].u.integer != 0) {
+    if(args>1 && TYPEOF(sp[1-args]) == T_INT && sp[1-args].u.integer != 0) {
       struct std8e_stor *s8;
       int i, c;
 
@@ -1522,8 +1585,17 @@ static ptrdiff_t feed_94(struct pike_string *str, struct std_cs_stor *s)
     p_wchar0 x = *p++;
     if(x<=0x20 || x>=0x7f)
       string_builder_putchar(&s->strbuild, x);
-    else if (table[x-0x21] != 0xe000)
-      string_builder_putchar(&s->strbuild, table[x-0x21]);
+    else {
+      UNICHAR uc = table[x-0x21];
+      if ((uc & 0xf800) == 0xd800) {
+	/* We use the surrogate block as an offset after the 94 table
+	 * to a NUL-terminated string of UNICHARs, for the case where
+	 * the mapping doesn't fit in a single UNICHAR. */
+	string_builder_utf16_strcat(&s->strbuild,
+				    table + 94 + (uc & 0x07ff));
+      } else if (uc != 0xe000)
+	string_builder_putchar(&s->strbuild, uc);
+    }
   }
   return 0;
 }
@@ -1543,8 +1615,17 @@ static ptrdiff_t feed_96(struct pike_string *str, struct std_cs_stor *s)
     p_wchar0 x = *p++;
     if(x<0xa0)
       string_builder_putchar(&s->strbuild, x);
-    else if (table[x-0xa0] != 0xe000)
-      string_builder_putchar(&s->strbuild, table[x-0xa0]);
+    else {
+      UNICHAR uc = table[x-0xa0];
+      if ((uc & 0xf800) == 0xd800) {
+	/* We use the surrogate block as an offset after the 96 table
+	 * to a NUL-terminated string of UNICHARs, for the case where
+	 * the mapping doesn't fit in a single UNICHAR. */
+	string_builder_utf16_strcat(&s->strbuild,
+				    table + 96 + (uc & 0x07ff));
+      } else if (uc != 0xe000)
+	string_builder_putchar(&s->strbuild, table[x-0xa0]);
+    }
   }
   return 0;
 }
@@ -1567,10 +1648,18 @@ static ptrdiff_t feed_9494(struct pike_string *str, struct std_cs_stor *s)
     else if(l==0)
       return 1;
     else if((y=(*p)&0x7f)>0x20 && y<0x7f) {
+      UNICHAR uc;
       --l;
       p++;
-      if (table[(x-0x21)*94+(y-0x21)] != 0xe000) {
-	string_builder_putchar(&s->strbuild, table[(x-0x21)*94+(y-0x21)]);
+      uc = table[(x-0x21)*94+(y-0x21)];
+      if ((uc & 0xf800) == 0xd800) {
+	/* We use the surrogate block as an offset after the 9494 table
+	 * to a NUL-terminated string of UNICHARs, for the case where
+	 * the mapping doesn't fit in a single UNICHAR. */
+	string_builder_utf16_strcat(&s->strbuild,
+				    table + 94*94 + (uc & 0x07ff));
+      } else if (uc != 0xe000) {
+	string_builder_putchar(&s->strbuild, uc);
       }
     } else {
       string_builder_putchar(&s->strbuild, x);
@@ -1597,9 +1686,17 @@ static ptrdiff_t feed_9696(struct pike_string *str, struct std_cs_stor *s)
     else if(l==0)
       return 1;
     else if((y=(*p)&0x7f)>=0x20) {
+      UNICHAR uc;
       --l;
       p++;
-      if (table[(x-0x20)*96+(y-0x20)] != 0xe000) {
+      uc = table[(x-0x20)*96+(y-0x20)];
+      if ((uc & 0xf800) == 0xd800) {
+	/* We use the surrogate block as an offset after the 9696 table
+	 * to a NUL-terminated string of UNICHARs, for the case where
+	 * the mapping doesn't fit in a single UNICHAR. */
+	string_builder_utf16_strcat(&s->strbuild,
+				    table + 96*96 + (uc & 0x07ff));
+      } else if (uc != 0xe000) {
 	string_builder_putchar(&s->strbuild, table[(x-0x20)*96+(y-0x20)]);
       }
     } else {
@@ -1748,7 +1845,7 @@ static void f_feed_utf8e(INT32 args)
   struct pike_string *str;
   struct std_cs_stor *cs = (struct std_cs_stor *)fp->current_storage;
 
-  get_all_args("feed()", args, "%W", &str);
+  get_all_args("feed", args, "%W", &str);
 
   feed_utf8e(cs, &cs->strbuild, str, cs->replace, MKREPCB(cs->repcb));
 
@@ -1884,7 +1981,7 @@ static void f_feed_utf_ebcdice(INT32 args)
   struct pike_string *str;
   struct std_cs_stor *cs = (struct std_cs_stor *)fp->current_storage;
 
-  get_all_args("feed()", args, "%W", &str);
+  get_all_args("feed", args, "%W", &str);
 
   feed_utf_ebcdice(cs, &cs->strbuild, str, cs->replace, MKREPCB(cs->repcb));
 
@@ -1957,7 +2054,7 @@ static void f_feed_utf7_5e(INT32 args)
   struct pike_string *str;
   struct std_cs_stor *cs = (struct std_cs_stor *)fp->current_storage;
 
-  get_all_args("feed()", args, "%W", &str);
+  get_all_args("feed", args, "%W", &str);
 
   feed_utf7_5e(cs, &cs->strbuild, str, cs->replace, MKREPCB(cs->repcb));
 
@@ -2117,7 +2214,7 @@ static void f_feed_utf7e(INT32 args)
   struct pike_string *str;
   struct std_cs_stor *cs = (struct std_cs_stor *)fp->current_storage;
 
-  get_all_args("feed()", args, "%W", &str);
+  get_all_args("feed", args, "%W", &str);
 
   feed_utf7e((struct utf7_stor *)(((char*)fp->current_storage)+utf7_stor_offs),
 	     &cs->strbuild, str, cs->replace, MKREPCB(cs->repcb));
@@ -2144,7 +2241,7 @@ static void f_drain_utf7e(INT32 args)
   f_drain(args);
 }
 
-static void std_8bite_init_stor(struct object *o)
+static void std_8bite_init_stor(struct object *UNUSED(o))
 {
   struct std8e_stor *s8 =
     (struct std8e_stor *)(fp->current_storage+std8e_stor_offs);
@@ -2156,7 +2253,7 @@ static void std_8bite_init_stor(struct object *o)
   s8->zero_char = 0xfffd;
 }
 
-static void std_8bite_exit_stor(struct object *o)
+static void std_8bite_exit_stor(struct object *UNUSED(o))
 {
   struct std8e_stor *s8 =
     (struct std8e_stor *)(fp->current_storage+std8e_stor_offs);
@@ -2230,7 +2327,7 @@ static void f_feed_std8e(INT32 args)
   struct pike_string *str;
   struct std_cs_stor *cs = (struct std_cs_stor *)fp->current_storage;
 
-  get_all_args("feed()", args, "%W", &str);
+  get_all_args("feed", args, "%W", &str);
 
   feed_std8e((struct std8e_stor *)(((char*)fp->current_storage)+
 				   std8e_stor_offs),
@@ -2240,7 +2337,7 @@ static void f_feed_std8e(INT32 args)
   push_object(this_object());
 }
 
-static void std_16bite_init_stor(struct object *o)
+static void std_16bite_init_stor(struct object *UNUSED(o))
 {
   struct std16e_stor *s16 =
     (struct std16e_stor *)(fp->current_storage+std16e_stor_offs);
@@ -2252,7 +2349,7 @@ static void std_16bite_init_stor(struct object *o)
   s16->sshift = 0;
 }
 
-static void std_16bite_exit_stor(struct object *o)
+static void std_16bite_exit_stor(struct object *UNUSED(o))
 {
   struct std16e_stor *s16 =
     (struct std16e_stor *)(fp->current_storage+std16e_stor_offs);
@@ -2339,7 +2436,7 @@ static void f_feed_std16e(INT32 args)
   struct pike_string *str;
   struct std_cs_stor *cs = (struct std_cs_stor *)fp->current_storage;
 
-  get_all_args("feed()", args, "%W", &str);
+  get_all_args("feed", args, "%W", &str);
 
   feed_std16e((struct std16e_stor *)(((char*)fp->current_storage)+
 				     std16e_stor_offs),
@@ -2352,8 +2449,32 @@ static void f_feed_std16e(INT32 args)
 
 PIKE_MODULE_INIT
 {
-  int i;
+  int i,n;
   struct svalue prog;
+  static p_wchar1 doubles_first_char[] =  { 0x0308, 0x0313, 0x0314 };
+  static p_wchar1 doubles_second_char[] = { 0x0300, 0x0301, 0x0342 };
+  p_wchar1 double_char[2];
+
+  /* Handling of double non-spacing characters used by eg ISO-IR-31. */
+  n = 0;
+  for(i = 0xe100; i < 0xe130; i += 0x0010) {
+    int j;
+    for(j = 0; j < 3; j++,n++) {
+      p_wchar1 c = i+j;
+      push_string(make_shared_binary_string1(&c, 1));
+    }
+  }
+  double_custom_chars = aggregate_array(n);
+  n = 0;
+  for (i = 0; i < 3; i++) {
+    int j;
+    double_char[0] = doubles_first_char[i];
+    for (j = 0; j < 3; j++,n++) {
+      double_char[1] = doubles_second_char[j];
+      push_string(make_shared_binary_string1(double_char, 2));
+    }
+  }
+  double_combiner_chars = aggregate_array(n);
 
   iso2022_init();
 
@@ -2373,9 +2494,7 @@ PIKE_MODULE_INIT
   set_exit_callback(exit_stor);
   std_cs_program = end_program();
 
-  prog.type = T_PROGRAM;
-  prog.subtype = 0;
-  prog.u.program = std_cs_program;
+  SET_SVAL(prog, T_PROGRAM, 0, program, std_cs_program);
 
   memset(rev64t, -1, sizeof(rev64t));
   for(i=0; i<64; i++)
@@ -2661,6 +2780,9 @@ PIKE_MODULE_EXIT
 
   iso2022_exit();
 
-  if (encode_err_prog.type != T_INT) free_svalue (&encode_err_prog);
-  if (decode_err_prog.type != T_INT) free_svalue (&decode_err_prog);
+  if (TYPEOF(encode_err_prog) != T_INT) free_svalue (&encode_err_prog);
+  if (TYPEOF(decode_err_prog) != T_INT) free_svalue (&decode_err_prog);
+
+  free_array(double_custom_chars);
+  free_array(double_combiner_chars);
 }
