@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: b7d3b2ee6cd33d6237a5b2f0bc89e17ad9e0dc2f $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -23,7 +22,7 @@
 #define sp Pike_sp
 #define fp Pike_fp
 
-#define PRGM_NAME "Locale.Charset.ISO2022"
+#define PRGM_NAME "Charset.ISO2022"
 
 
 static struct program *iso2022dec_program = NULL;
@@ -57,8 +56,10 @@ struct iso2022enc_stor {
 
 #define EMIT(X) string_builder_putchar(&s->strbuild,(X))
 
-static const UNICHAR * const *transltab[4] = { iso2022_94, iso2022_96,
-					       iso2022_9494, iso2022_9696 };
+static const UNICHAR * const *transltab[8] = { iso2022_94, iso2022_96,
+					       iso2022_9494, iso2022_9696,
+					       iso2022_94_2, NULL,
+					       NULL, NULL, };
 
 static ptrdiff_t eat_text(unsigned char *src, ptrdiff_t srclen,
 			  struct iso2022_stor *s, struct gdesc *g)
@@ -109,16 +110,33 @@ static ptrdiff_t eat_text(unsigned char *src, ptrdiff_t srclen,
       char x = (*src++)&0x7f;
       if(x==0x20 || x==0x7f)
 	EMIT((UNICHAR)x);
-      else if (g->transl[x-0x21] != 0xe000)
-	EMIT(g->transl[x-0x21]);
+      else {
+	UNICHAR uc = g->transl[x-0x21];
+	if ((uc & 0xf800) == 0xd800) {
+	  /* We use the surrogate block as an offset after the 94 table
+	   * to a NUL-terminated string of UNICHARs, for the case where
+	   * the mapping doesn't fit in a single UNICHAR. */
+	  string_builder_utf16_strcat(&s->strbuild,
+				      g->transl + 94 + (uc & 0x07ff));
+	} else if (uc != 0xe000)
+	  EMIT(uc);
+      }
     }
     return 0;
     break;
   case MODE_96:
     while(srclen--) {
       UNICHAR c;
-      if ((c = g->transl[((*src++)&0x7f)-0x20]) != 0xe000)
-	EMIT(c);
+      if ((c = g->transl[((*src++)&0x7f)-0x20]) != 0xe000) {
+	if ((c & 0xf800) == 0xd800) {
+	  /* We use the surrogate block as an offset after the 96 table
+	   * to a NUL-terminated string of UNICHARs, for the case where
+	   * the mapping doesn't fit in a single UNICHAR. */
+	  string_builder_utf16_strcat(&s->strbuild,
+				      g->transl + 96 + (c & 0x07ff));
+	} else
+	  EMIT(c);
+      }
     }
     return 0;
     break;
@@ -127,8 +145,15 @@ static ptrdiff_t eat_text(unsigned char *src, ptrdiff_t srclen,
       char hi, lo;
       if ((hi = (*src++)&0x7f)!=0x20 && hi != 0x7f &&
 	  (lo = (*src)&0x7f)!=0x20 && lo != 0x7f) {
-	if (g->transl[(hi-0x21)*94+(lo-0x21)] != 0xe000)
-	  EMIT(g->transl[(hi-0x21)*94+(lo-0x21)]);
+	UNICHAR uc = g->transl[(hi-0x21)*94+(lo-0x21)];
+	if ((uc & 0xf800) == 0xd800) {
+	  /* We use the surrogate block as an offset after the 9494 table
+	   * to a NUL-terminated string of UNICHARs, for the case where
+	   * the mapping doesn't fit in a single UNICHAR. */
+	  string_builder_utf16_strcat(&s->strbuild,
+				      g->transl + 94*94 + (uc & 0x07ff));
+	} else if (uc != 0xe000)
+	  EMIT(uc);
 	src++;
 	srclen -= 2;
       } else {
@@ -145,8 +170,15 @@ static ptrdiff_t eat_text(unsigned char *src, ptrdiff_t srclen,
     while(srclen>1) {
       char hi = (*src++)&0x7f;
       char lo = (*src++)&0x7f;
-      if (g->transl[(hi-0x20)*96+(lo-0x20)] != 0xe000)
-	EMIT(g->transl[(hi-0x20)*96+(lo-0x20)]);
+      UNICHAR uc = g->transl[(hi-0x20)*96+(lo-0x20)];
+      if ((uc & 0xf800) == 0xd800) {
+	/* We use the surrogate block as an offset after the 9696 table
+	 * to a NUL-terminated string of UNICHARs, for the case where
+	 * the mapping doesn't fit in a single UNICHAR. */
+	string_builder_utf16_strcat(&s->strbuild,
+				    g->transl + 96*96 + (uc & 0x07ff));
+      } else if (uc != 0xe000)
+	EMIT(uc);
       srclen -= 2;
     }
     break;
@@ -157,7 +189,7 @@ static ptrdiff_t eat_text(unsigned char *src, ptrdiff_t srclen,
 static INT32 parse_esc(unsigned char *src, ptrdiff_t srclen,
 		       struct iso2022_stor *s)
 {
-  int grp=-1, wide=0, final, mode, l=1;
+  int grp=-1, wide=0, final, mode, l=1, second = 0;
   struct gdesc *g;
 
   /* Return value:
@@ -214,7 +246,7 @@ static INT32 parse_esc(unsigned char *src, ptrdiff_t srclen,
       return -1;
     }
 
-  if(src[1]<0x24 || src[1]>=0x30)
+  if((src[1]<0x24 || src[1]>=0x30) && src[1] != 0x21)
     return -1;
 
   if(srclen<3)
@@ -227,11 +259,43 @@ static INT32 parse_esc(unsigned char *src, ptrdiff_t srclen,
   }
   if(src[l] >= 0x28 && src[l] < 0x30)
     grp = src[l++]-0x28;
+  /* FIXME: Consider supporting the general coding modes:
+   *
+   *   <ESC> 0x25 0x40		ISO-2022, ISO-IR 108, ISO-IR 178.
+   *   <ESC> 0x25 0x41		CSA T500/ANSI X3.110-1983 (NAPLPS), ISO-IR 108.
+   *   <ESC> 0x25 0x42		UTF-1, ISO-IR 178.
+   *   <ESC> 0x25 0x43		CCITT T.101, Data Syntax I, ISO-IR 131.
+   *   <ESC> 0x25 0x44		CCITT T.101, Data Syntax II, ISO-IR 145.
+   *   <ESC> 0x25 0x45		CCITT T.101, Photo Data Syntax, ISO-IR 160.
+   *   <ESC> 0x25 0x46		CCITT T.101, Audio Data Syntax, ISO-IR 161.
+   *   <ESC> 0x25 0x47		UTF-8, std return, ISO-IR 196.
+   *   <ESC> 0x25 0x48		CCITT T.107, Videotex Data Syntax, ISO-IR 188.
+   *   <ESC> 0x25 0x2f 0x40	USC-2, level 1, ISO-IR 162.
+   *   <ESC> 0x25 0x2f 0x41	USC-4, level 1, ISO-IR 163.
+   *   <ESC> 0x25 0x2f 0x42	Virtual Terminal Service, ISO-IR 125.
+   *   <ESC> 0x25 0x2f 0x43	UCS-2, level 2, ISO-IR 174.
+   *   <ESC> 0x25 0x2f 0x44	UCS-4, level 2, ISO-IR 175.
+   *   <ESC> 0x25 0x2f 0x45	UCS-2, level 3, ISO-IR 176.
+   *   <ESC> 0x25 0x2f 0x46	UCS-4, level 4, ISO-IR 177.
+   *   <ESC> 0x25 0x2f 0x47	UTF-8, level 1, no std return, ISO-IR 190.
+   *   <ESC> 0x25 0x2f 0x48	UTF-8, level 2, no std return, ISO-IR 191.
+   *   <ESC> 0x25 0x2f 0x49	UTF-8, level 3, no std return, ISO-IR 192.
+   *   <ESC> 0x25 0x2f 0x4a	UTF-16, level 1, no std return, ISO-IR-193.
+   *   <ESC> 0x25 0x2f 0x4b	UTF-16, level 2, no std return, ISO-IR-194.
+   *   <ESC> 0x25 0x2f 0x4c	UTF-16, level 3, no std return, ISO-IR-195.
+   *   <ESC> 0x64		CODING METHOD DELIMITER, ISO-IR 189.
+   */
   if(l>=srclen)
     return 0;
-  if(grp<0 || src[l]<0x30 || src[l]>0x7e)
+  if(grp<0 || ((src[l]<0x30 || src[l]>0x7e) && src[l] != 0x21))
     return -1;
   final = src[l++];
+  if (final == 0x21) {
+    /* Second set of 94-character maps. */
+    if (l >= srclen) return 0;
+    second = 4;
+    final = src[l++];
+  }
   mode = MODE_94;
   if(grp>=4) {
     mode = MODE_96;
@@ -245,7 +309,7 @@ static INT32 parse_esc(unsigned char *src, ptrdiff_t srclen,
   if(final<0x40 || (wide && final>=0x60))
     g->transl = NULL;
   else
-    g->transl = transltab[mode][final-0x40];
+    g->transl = transltab[mode + second][final-0x40];
   return l;
 }
 
@@ -350,7 +414,7 @@ static int call_repcb(struct svalue *repcb, p_wchar2 ch)
 {
   push_string(make_shared_binary_string2(&ch, 1));
   apply_svalue(repcb, 1);
-  if(sp[-1].type == T_STRING)
+  if(TYPEOF(sp[-1]) == T_STRING)
     return 1;
   pop_stack();
   return 0;
@@ -1089,8 +1153,7 @@ static void eat_enc_string(struct pike_string *str, struct iso2022enc_stor *s,
 	  if(index!=0 && (ttab = transltab[mode][index-0x10])!=NULL) {
 	    switch(mode) {
 	    case MODE_94:
-	      rmap = (p_wchar1 *)xalloc((0x10000-0x100)*sizeof(p_wchar1));
-	      memset(rmap, 0, (0x10000-0x100)*sizeof(p_wchar1));
+	      rmap = xcalloc(0x10000-0x100, sizeof(p_wchar1));
 	      for(ch=0; ch<94; ch++)
 		if(ttab[ch]>=0x100 && ttab[ch]!=0xfffd)
 		  rmap[ttab[ch]-0x100]=ch+33;
@@ -1110,8 +1173,7 @@ static void eat_enc_string(struct pike_string *str, struct iso2022enc_stor *s,
 		ttab = NULL;
 	      break;
 	    case MODE_96:
-	      rmap = (p_wchar1 *)xalloc((0x10000-0x100)*sizeof(p_wchar1));
-	      memset(rmap, 0, (0x10000-0x100)*sizeof(p_wchar1));
+	      rmap = xcalloc(0x10000-0x100, sizeof(p_wchar1));
 	      for(ch=0; ch<96; ch++)
 		if(ttab[ch]>=0x100 && ttab[ch]!=0xfffd)
 		  rmap[ttab[ch]-0x100]=ch+32;
@@ -1131,8 +1193,7 @@ static void eat_enc_string(struct pike_string *str, struct iso2022enc_stor *s,
 		ttab = NULL;
 	      break;
 	    case MODE_9494:
-	      rmap = (p_wchar1 *)xalloc((0x10000-0x100)*sizeof(p_wchar1));
-	      memset(rmap, 0, (0x10000-0x100)*sizeof(p_wchar1));
+	      rmap = xcalloc(0x10000-0x100, sizeof(p_wchar1));
 	      for(ttt=ttab, ch=0; ch<94; ch++)
 		for(ch2=0; ch2<94; ch2++, ttt++)
 		if(*ttt>=0x100 && *ttt!=0xfffd)
@@ -1159,8 +1220,7 @@ static void eat_enc_string(struct pike_string *str, struct iso2022enc_stor *s,
 		ttab = NULL;
 	      break;
 	    case MODE_9696:
-	      rmap = (p_wchar1 *)xalloc((0x10000-0x100)*sizeof(p_wchar1));
-	      memset(rmap, 0, (0x10000-0x100)*sizeof(p_wchar1));
+	      rmap = xcalloc(0x10000-0x100, sizeof(p_wchar1));
 	      for(ttt=ttab, ch=0; ch<96; ch++)
 		for(ch2=0; ch2<96; ch2++, ttt++)
 		if(*ttt>=0x100 && *ttt!=0xfffd)
@@ -1205,7 +1265,7 @@ static void f_feed(INT32 args)
 {
   struct pike_string *str;
 
-  get_all_args(PRGM_NAME"Dec->feed()", args, "%S", &str);
+  get_all_args("feed", args, "%S", &str);
 
   eat_string(str, (struct iso2022_stor *)fp->current_storage);
 
@@ -1260,6 +1320,8 @@ static void f_drain(INT32 args)
 	}
       }
       break;
+    case 0: /* Dummy case to silence clang warning */
+      break;
     }
   }
 
@@ -1311,11 +1373,11 @@ static void f_enc_feed(INT32 args)
 {
   struct pike_string *str;
 
-  get_all_args(PRGM_NAME"Enc->feed()", args, "%W", &str);
+  get_all_args("feed", args, "%W", &str);
   if( str->len )
     eat_enc_string(str, (struct iso2022enc_stor *)fp->current_storage,
 		   ((struct iso2022enc_stor *)fp->current_storage)->replace,
-		   (((struct iso2022enc_stor *)fp->current_storage)->repcb.type
+		   (TYPEOF(((struct iso2022enc_stor *)fp->current_storage)->repcb)
 		    == T_FUNCTION?
 		    &((struct iso2022enc_stor *)fp->current_storage)->repcb :
 		    NULL));
@@ -1398,13 +1460,13 @@ static void f_create(INT32 args)
 
   select_encoding_parameters(s, sp[-args].u.string);
 
-  if(args>1 && sp[1-args].type == T_STRING) {
+  if(args>1 && TYPEOF(sp[1-args]) == T_STRING) {
     if(s->replace != NULL)
       free_string(s->replace);
     add_ref(s->replace = sp[1-args].u.string);
   }
 
-  if(args>2 && sp[2-args].type == T_FUNCTION)
+  if(args>2 && TYPEOF(sp[2-args]) == T_FUNCTION)
     assign_svalue(&s->repcb, &sp[2-args]);
 
   pop_n_elems(args);
@@ -1424,7 +1486,7 @@ static void f_set_repcb(INT32 args)
   pop_n_elems(args);
 }
 
-static void init_stor(struct object *o)
+static void init_stor(struct object *UNUSED(o))
 {
   struct iso2022_stor *s = (struct iso2022_stor *)fp->current_storage;
 
@@ -1436,7 +1498,7 @@ static void init_stor(struct object *o)
   pop_n_elems(1);
 }
 
-static void exit_stor(struct object *o)
+static void exit_stor(struct object *UNUSED(o))
 {
   struct iso2022_stor *s = (struct iso2022_stor *)fp->current_storage;
 
@@ -1448,7 +1510,7 @@ static void exit_stor(struct object *o)
   free_string_builder(&s->strbuild);
 }
 
-static void init_enc_stor(struct object *o)
+static void init_enc_stor(struct object *UNUSED(o))
 {
   struct iso2022enc_stor *s = (struct iso2022enc_stor *)fp->current_storage;
   int i;
@@ -1463,7 +1525,7 @@ static void init_enc_stor(struct object *o)
   pop_n_elems(1);
 }
 
-static void exit_enc_stor(struct object *o)
+static void exit_enc_stor(struct object *UNUSED(o))
 {
   struct iso2022enc_stor *s = (struct iso2022enc_stor *)fp->current_storage;
   int i;
