@@ -26,6 +26,7 @@
 #include "module_support.h"
 #include "operators.h"
 #include "pike_types.h"
+#include "bignum.h"
 
 
 #include <stdio.h>
@@ -1205,6 +1206,7 @@ static void img_jpeg_decode(INT32 args,int mode)
    if (mode!=IMG_DECODE_HEADER)
    {
       int bytes_per_pixel;
+      INT32 bytes_needed;
 
       jpeg_start_decompress(&mds.cinfo);
       bytes_per_pixel = mds.cinfo.output_components;
@@ -1212,8 +1214,16 @@ static void img_jpeg_decode(INT32 args,int mode)
       o=clone_object(image_program,0);
       img=get_storage(o,image_program);
       if (!img) Pike_error("image no image? foo?\n"); /* should never happen */
-      img->img=malloc(bytes_per_pixel *
-		      mds.cinfo.output_width*mds.cinfo.output_height);
+      if (DO_INT32_MUL_OVERFLOW(sizeof(*img->img), mds.cinfo.output_width,
+			        &bytes_needed)
+	  || DO_INT32_MUL_OVERFLOW(bytes_needed, mds.cinfo.output_height,
+		                   &bytes_needed))
+      {
+	 jpeg_destroy((struct jpeg_common_struct*)&mds.cinfo);
+	 free_object(o);
+	 Pike_error("Image.JPEG.decode: Image too large (>%d bytes).\n", MAX_INT32);
+      }
+      img->img=malloc(bytes_needed);
       if (!img->img)
       {
 	 jpeg_destroy((struct jpeg_common_struct*)&mds.cinfo);
