@@ -1,7 +1,6 @@
 #include "global.h"
 #include "stralloc.h"
 #include "global.h"
-RCSID("$Id: dd84289aad0a4befb519ee80e4465b1219b687e8 $");
 #include "pike_macros.h"
 #include "interpret.h"
 #include "program.h"
@@ -114,7 +113,7 @@ static void f_blobs_add_words( INT32 args )
 		&docid, &words, &field_id);
 
   for( i = 0; i<words->size; i++ )
-    if( words->item[i].type != PIKE_T_STRING )
+    if( TYPEOF(words->item[i]) != PIKE_T_STRING )
       Pike_error("Illegal element %d in words array\n", i );
     else
     {
@@ -129,7 +128,7 @@ static void f_blobs_add_words( INT32 args )
 	wf_buffer_wbyte( x->buffer, 0 );
 	x->word_data_offset = x->buffer->size-1;
       }
-      if( x->buffer->data[x->word_data_offset] < 255 )
+      if( (unsigned char)x->buffer->data[x->word_data_offset] < 255 )
       {
 	unsigned short s;
 	x->buffer->data[x->word_data_offset]++;
@@ -171,10 +170,8 @@ static void f_blobs_read( INT32 args )
   {
     if( t->next_ind >= HSIZE )
     {
-      a->item[0].type = PIKE_T_INT;
-      a->item[0].u.integer = 0;
-      a->item[1].type = PIKE_T_INT;
-      a->item[1].u.integer = 0;
+      SET_SVAL(a->item[0], PIKE_T_INT, NUMBER_NUMBER, integer, 0);
+      SET_SVAL(a->item[1], PIKE_T_INT, NUMBER_NUMBER, integer, 0);
       push_array( a );
       return;
     }
@@ -182,11 +179,10 @@ static void f_blobs_read( INT32 args )
     t->next_ind++;
   }
 
-  a->item[0].type = PIKE_T_STRING;
-  a->item[0].u.string = t->next_h->id;
-  a->item[1].type = PIKE_T_STRING;
-  a->item[1].u.string = make_shared_binary_string( t->next_h->buffer->data,
-						   t->next_h->buffer->size );
+  SET_SVAL(a->item[0], PIKE_T_STRING, 0, string, t->next_h->id);
+  SET_SVAL(a->item[1], PIKE_T_STRING, 0, string,
+	   make_shared_binary_string( t->next_h->buffer->data,
+				      t->next_h->buffer->size ));
   wf_buffer_free( t->next_h->buffer );
   t->next_h->buffer = 0;
   t->next_h->id = 0;
@@ -214,7 +210,7 @@ static int compare_wordarrays( const void *_a, const void *_b )
  *! the blobs struct once. Also, once you have called @[read] or @[read_all_sorted],
  *! @[add_words] will no longer work as expected.
  */
-static void f_blobs_read_all_sorted( INT32 args )
+static void f_blobs_read_all_sorted( INT32 UNUSED(args) )
 {
   struct array *g = allocate_array( THIS->nwords );
   int i;
@@ -236,23 +232,23 @@ static void f_blobs_read_all_sorted( INT32 args )
 /*! @endmodule
  */
 
-static void init_blobs_struct( )
+static void init_blobs_struct(struct object *UNUSED(o))
 {
-  MEMSET( THIS, 0, sizeof( struct blobs ) );
+  memset( THIS, 0, sizeof( struct blobs ) );
   THIS->size = sizeof( struct blobs ) + 128;
 }
 
-static void exit_blobs_struct( )
+static void exit_blobs_struct(struct object *o)
 {
   int i;
   for( i = 0; i<HSIZE; i++ )
     if( THIS->hash[i] )
       free_hash( THIS->hash[i] );
-  init_blobs_struct();
+  init_blobs_struct(o);
 }
 
 static struct program *blobs_program;
-void init_blobs_program()
+void init_blobs_program(void)
 {
   start_new_program();
   ADD_STORAGE( struct blobs );
@@ -267,7 +263,7 @@ void init_blobs_program()
   add_program_constant( "Blobs", blobs_program, 0 );
 }
 
-void exit_blobs_program()
+void exit_blobs_program(void)
 {
   free_program( blobs_program );
 }
