@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: ac9178d77e353d07f73751cf8cf247cd27a60edc $
 */
 
 #ifndef MULTISET_H
@@ -18,10 +17,6 @@
 #include "svalue.h"
 #include "dmalloc.h"
 #include "rbtree.h"
-#include "block_alloc_h.h"
-
-/* Keep this defined so that code can test which multiset API is in use. */
-#define PIKE_NEW_MULTISETS
 
 /* Note: Don't access the ind svalue (or at least not its type field)
  * in the following directly, since the rbtree flags overlay that. Use
@@ -54,9 +49,6 @@ union msnode
 struct multiset_data
 {
   INT32 refs, noval_refs;
-#ifdef PIKE_RUN_UNLOCKED
-#error multiset_data has not been adapted for running unlocked.
-#endif
   union msnode *root, *free_list;
   struct svalue cmp_less;
   INT32 size, allocsize;
@@ -140,12 +132,12 @@ struct multiset
  *    remembered even after it has been deleted.
  */
 
-/* The following are compatible with PIKE_WEAK_INDICES and PIKE_WEAK_VALUES. */
 #define MULTISET_WEAK_INDICES	2
-#define MULTISET_WEAK_VALUES	4
-#define MULTISET_WEAK		6
+#define MULTISET_WEAK		2
 
-#define MULTISET_INDVAL		8
+#define MULTISET_WEAK_VALUES	0
+
+#define MULTISET_INDVAL		0
 
 extern struct multiset *first_multiset;
 extern struct multiset *gc_internal_multiset;
@@ -154,7 +146,7 @@ PMOD_EXPORT void multiset_clear_node_refs (struct multiset *l);
 
 #ifdef PIKE_DEBUG
 /* To get good type checking. */
-static INLINE union msnode *msnode_check (union msnode *x)
+static INLINE union msnode ATTRIBUTE((unused)) *msnode_check (union msnode *x)
   {return x;}
 PMOD_EXPORT extern const char msg_no_multiset_flag_marker[];
 #else
@@ -169,16 +161,16 @@ PMOD_EXPORT extern const char msg_no_multiset_flag_marker[];
 #define low_multiset_next(NODE) MULTISET_STEP_FUNC (rb_next, NODE)
 #define low_multiset_get_nth(MSD, N)					\
   ((union msnode *) rb_get_nth ((struct rb_node_hdr *) (MSD)->root, (N)))
-union msnode *low_multiset_find_eq (struct multiset *l, struct svalue *key);
 
 #define low_assign_multiset_index_no_free(TO, NODE) do {		\
     struct svalue *_ms_index_to_ = (TO);				\
     *_ms_index_to_ = msnode_check (NODE)->i.ind;			\
     DO_IF_DEBUG (							\
-      if (!(_ms_index_to_->type & MULTISET_FLAG_MARKER))		\
+      if (!(TYPEOF(*_ms_index_to_) & MULTISET_FLAG_MARKER))		\
 	Pike_fatal (msg_no_multiset_flag_marker);			\
     );									\
-    _ms_index_to_->type &= ~MULTISET_FLAG_MASK;				\
+    SET_SVAL_TYPE(*_ms_index_to_,					\
+		  TYPEOF(*_ms_index_to_) & ~MULTISET_FLAG_MASK);	\
     add_ref_svalue (_ms_index_to_);					\
   } while (0)
 #define low_assign_multiset_index(TO, NODE) do {			\
@@ -190,9 +182,9 @@ union msnode *low_multiset_find_eq (struct multiset *l, struct svalue *key);
   low_assign_multiset_index_no_free (Pike_sp++, (NODE))
 #define low_use_multiset_index(NODE, VAR)				\
   ((VAR) = msnode_check (NODE)->i.ind,					\
-   DO_IF_DEBUG ((VAR).type & MULTISET_FLAG_MARKER ? 0 :			\
+    DO_IF_DEBUG (TYPEOF(VAR) & MULTISET_FLAG_MARKER ? 0 :		\
 		(Pike_fatal (msg_no_multiset_flag_marker), 0) COMMA)	\
-   (VAR).type &= ~MULTISET_FLAG_MASK,					\
+   SET_SVAL_TYPE((VAR), TYPEOF(VAR) & ~MULTISET_FLAG_MASK),		\
    &(VAR))
 
 #define low_get_multiset_value(MSD, NODE)				\
@@ -230,7 +222,8 @@ PMOD_EXPORT INT32 multiset_sizeof (struct multiset *l);
 #define free_multiset(M) do_free_multiset (M)
 #else
 
-void really_free_multiset (struct multiset *l);
+PMOD_EXPORT void really_free_multiset (struct multiset *l);
+PMOD_EXPORT void count_memory_in_multisets (size_t * n, size_t * s);
 
 #define free_multiset(L) do {						\
     struct multiset *_ms_ = (L);					\
@@ -261,8 +254,6 @@ PMOD_EXPORT union msnode *debug_check_msnode (
 
 #endif
 
-BLOCK_ALLOC_FILL_PAGES (multiset, 2);
-
 /* See rbtree.h for a description of the operations.
  *
  * If cmp_less is used, it's a function pointer used as `< to compare
@@ -308,11 +299,6 @@ BLOCK_ALLOC_FILL_PAGES (multiset, 2);
  */
 
 /* Returns the node offset, or -1 if no match was found. */
-PMOD_EXPORT ptrdiff_t multiset_find_eq (struct multiset *l, struct svalue *key);
-PMOD_EXPORT ptrdiff_t multiset_find_lt (struct multiset *l, struct svalue *key);
-PMOD_EXPORT ptrdiff_t multiset_find_gt (struct multiset *l, struct svalue *key);
-PMOD_EXPORT ptrdiff_t multiset_find_le (struct multiset *l, struct svalue *key);
-PMOD_EXPORT ptrdiff_t multiset_find_ge (struct multiset *l, struct svalue *key);
 PMOD_EXPORT ptrdiff_t multiset_first (struct multiset *l);
 PMOD_EXPORT ptrdiff_t multiset_last (struct multiset *l);
 PMOD_EXPORT ptrdiff_t multiset_prev (struct multiset *l, ptrdiff_t nodepos);
@@ -349,7 +335,7 @@ PMOD_EXPORT int msnode_is_deleted (struct multiset *l, ptrdiff_t nodepos);
   assign_multiset_index_no_free (Pike_sp++, (L), (NODEPOS))
 #define use_multiset_index(L, NODEPOS, VAR)				\
   ((VAR) = access_msnode ((L), (NODEPOS))->i.ind,			\
-   (VAR).type &= ~MULTISET_FLAG_MASK,					\
+   SET_SVAL_TYPE((VAR), TYPEOF(VAR) & ~MULTISET_FLAG_MASK),		\
    &(VAR))
 
 #define get_multiset_value(L, NODEPOS)					\
@@ -381,42 +367,22 @@ PMOD_EXPORT struct multiset *real_allocate_multiset (int allocsize,
 PMOD_EXPORT void do_free_multiset (struct multiset *l);
 PMOD_EXPORT void multiset_fix_type_field (struct multiset *l);
 PMOD_EXPORT void multiset_set_flags (struct multiset *l, int flags);
-PMOD_EXPORT void multiset_set_cmp_less (struct multiset *l,
-					struct svalue *cmp_less);
 PMOD_EXPORT struct multiset *mkmultiset (struct array *indices);
-PMOD_EXPORT struct multiset *mkmultiset_2 (struct array *indices,
-					   struct array *values,
-					   struct svalue *cmp_less);
 PMOD_EXPORT void multiset_insert (struct multiset *l,
 				  struct svalue *ind);
-PMOD_EXPORT ptrdiff_t multiset_insert_2 (struct multiset *l,
-					 struct svalue *ind,
-					 struct svalue *val,
-					 int replace);
-PMOD_EXPORT ptrdiff_t multiset_add (struct multiset *l,
-				    struct svalue *ind,
-				    struct svalue *val);
-PMOD_EXPORT ptrdiff_t multiset_add_after (struct multiset *l,
-					  ptrdiff_t node,
-					  struct svalue *ind,
-					  struct svalue *val);
 PMOD_EXPORT int multiset_delete (struct multiset *l,
 				 struct svalue *ind);
+#ifdef MULTISET_INDVAL
 PMOD_EXPORT int multiset_delete_2 (struct multiset *l,
 				   struct svalue *ind,
 				   struct svalue *removed_val);
-PMOD_EXPORT void multiset_delete_node (struct multiset *l,
-				       ptrdiff_t node);
-PMOD_EXPORT int multiset_member (struct multiset *l,
-				 struct svalue *key);
 PMOD_EXPORT struct svalue *multiset_lookup (struct multiset *l,
 					    struct svalue *key);
+#endif
+PMOD_EXPORT int multiset_member (struct multiset *l,
+				 struct svalue *key);
 struct array *multiset_indices (struct multiset *l);
 struct array *multiset_values (struct multiset *l);
-struct array *multiset_range_indices (struct multiset *l,
-				      ptrdiff_t beg, ptrdiff_t end);
-struct array *multiset_range_values (struct multiset *l,
-				     ptrdiff_t beg, ptrdiff_t end);
 PMOD_EXPORT void check_multiset_for_destruct (struct multiset *l);
 PMOD_EXPORT struct multiset *copy_multiset (struct multiset *l);
 PMOD_EXPORT struct multiset *merge_multisets (struct multiset *a,
@@ -426,7 +392,6 @@ PMOD_EXPORT struct multiset *add_multisets (struct svalue *argp, int count);
 PMOD_EXPORT int multiset_equal_p (struct multiset *a, struct multiset *b,
 				  struct processing *p);
 void describe_multiset (struct multiset *l, struct processing *p, int indent);
-void simple_describe_multiset (struct multiset *l);
 int multiset_is_constant (struct multiset *l, struct processing *p);
 node *make_node_from_multiset (struct multiset *l);
 PMOD_EXPORT void f_aggregate_multiset (int args);
@@ -434,7 +399,7 @@ struct multiset *copy_multiset_recursively (struct multiset *l,
 					    struct mapping *p);
 PMOD_EXPORT ptrdiff_t multiset_get_nth (struct multiset *l, size_t n);
 
-PMOD_EXPORT void visit_multiset (struct multiset *l, int action);
+PMOD_EXPORT void visit_multiset (struct multiset *l, int action, void *extra);
 unsigned gc_touch_all_multisets (void);
 void gc_check_all_multisets (void);
 void gc_mark_multiset_as_referenced (struct multiset *l);
@@ -444,9 +409,9 @@ void real_gc_cycle_check_multiset (struct multiset *l, int weak);
 void gc_cycle_check_all_multisets (void);
 size_t gc_free_all_unreferenced_multisets (void);
 
-#define visit_multiset_ref(L, REF_TYPE)				\
+#define visit_multiset_ref(L, REF_TYPE, EXTRA)			\
   visit_ref (pass_multiset (L), (REF_TYPE),			\
-	     (visit_thing_fn *) &visit_multiset, NULL)
+	     (visit_thing_fn *) &visit_multiset, (EXTRA))
 #define gc_cycle_check_multiset(X, WEAK) \
   gc_cycle_enqueue ((gc_cycle_check_cb *) real_gc_cycle_check_multiset, (X), (WEAK))
 
