@@ -163,25 +163,27 @@ PMOD_EXPORT extern const char msg_unsetjmp_nosync_2[];
 
 
 #ifdef PIKE_DEBUG
-#define SET_ONERROR(X,Y,Z)					\
-  do{								\
-    check_recovery_context();					\
-    OED_FPRINTF((stderr, "SET_ONERROR(%p, %p, %p) %s:%d\n",	\
-		 &(X), (Y), (void *)(Z), __FILE__, __LINE__));	\
-    X.frame_pointer = Pike_interpreter.frame_pointer;		\
-    X.func=(error_call)(Y);					\
-    DO_IF_DMALLOC( if( X.func == free ) X.func=dmalloc_free);	\
-    X.arg=(void *)(Z);						\
-    if(!Pike_interpreter.recoveries) {				\
-      X.previous = NULL;					\
-      break;							\
-    }								\
-    X.previous=Pike_interpreter.recoveries->onerror;		\
-    X.file = __FILE__;						\
-    X.line = __LINE__;						\
-    Pike_interpreter.recoveries->onerror=&X;			\
+#define LOW_SET_ONERROR(X,Y,Z)						\
+  do{									\
+    check_recovery_context();						\
+    OED_FPRINTF((stderr, "SET_ONERROR(%p, %p, %p) %s:%d\n",		\
+		 (X), (Y), (void *)(Z), __FILE__, __LINE__));		\
+    (X)->frame_pointer = Pike_interpreter.frame_pointer;		\
+    (X)->func = (error_call)(Y);					\
+    DO_IF_DMALLOC( if( (X)->func == free ) (X)->func = dmalloc_free);	\
+    (X)->arg = (void *)(Z);						\
+    if (!Pike_interpreter.recoveries) {					\
+      (X)->previous = NULL;						\
+      break;								\
+    }									\
+    (X)->previous = Pike_interpreter.recoveries->onerror;		\
+    (X)->file = __FILE__;						\
+    (X)->line = __LINE__;						\
+    Pike_interpreter.recoveries->onerror = (X);				\
   }while(0)
 
+#define SET_ONERROR(X,Y,Z) LOW_SET_ONERROR(&X, Y, Z)
+
 PMOD_EXPORT extern const char msg_last_setjmp[];
 PMOD_EXPORT extern const char msg_unset_onerr_nosync_1[];
 PMOD_EXPORT extern const char msg_unset_onerr_nosync_2[];
@@ -214,19 +216,21 @@ PMOD_EXPORT extern const char msg_assert_onerr[];
     } \
   }while(0)
 #else /* !PIKE_DEBUG */
-#define SET_ONERROR(X,Y,Z) \
+#define LOW_SET_ONERROR(X,Y,Z) \
   do{ \
-     X.func=(error_call)(Y); \
-     X.arg=(void *)(Z); \
-     X.frame_pointer = Pike_interpreter.frame_pointer; \
+     (X)->func=(error_call)(Y); \
+     (X)->arg=(void *)(Z); \
+     (X)->frame_pointer = Pike_interpreter.frame_pointer; \
      if(!Pike_interpreter.recoveries) {		       \
-       X.previous = NULL;			       \
+       (X)->previous = NULL;			       \
        break;					       \
      }						       \
-     X.previous=Pike_interpreter.recoveries->onerror; \
-     Pike_interpreter.recoveries->onerror=&X; \
+     (X)->previous=Pike_interpreter.recoveries->onerror; \
+     Pike_interpreter.recoveries->onerror=(X); \
   }while(0)
 
+#define SET_ONERROR(X,Y,Z) LOW_SET_ONERROR(&X, Y, Z)
+
 #define UNSET_ONERROR(X) Pike_interpreter.recoveries && (Pike_interpreter.recoveries->onerror=X.previous)
 
 #define ASSERT_ONERROR(X)
