@@ -1,6 +1,4 @@
 /*
- * $Id: 9a642d28809b2339d9c104ca751e0e94b521c357 $
- *
  * Pike interface to Common Object Model (COM)
  *
  * Tomas Nilsson
@@ -30,6 +28,7 @@
 #include "gc.h"
 #include "threads.h"
 #include "operators.h"
+#include "pike_types.h"
 
 #ifdef HAVE_COM
 
@@ -165,7 +164,7 @@ static void set_variant_arg(VARIANT *v, struct svalue *sv)
 
   VariantInit(v);
       
-  switch(sv->type)
+  switch(TYPEOF(*sv))
   {
     case PIKE_T_INT:
       v->vt = VT_I4;
@@ -218,22 +217,22 @@ static void set_variant_arg(VARIANT *v, struct svalue *sv)
       /* TODO: Convert array to SAFEARRAY ? */
     case PIKE_T_ARRAY:
       Pike_error("Com: create_variant: Pike array can't be converted!\n",
-                 sv->type);
+                 TYPEOF(*sv));
       break;
 
     case PIKE_T_MAPPING:
       Pike_error("Com: create_variant: Pike mapping can't be converted!\n",
-                 sv->type);
+                 TYPEOF(*sv));
       break;
 
     case PIKE_T_MULTISET:
       Pike_error("Com: create_variant: Pike multiset can't be converted!\n",
-                 sv->type);
+                 TYPEOF(*sv));
       break;
 
     default:
       Pike_error("Com: create_variant: Pike type %d can't be converted!\n",
-                 sv->type);
+                 TYPEOF(*sv));
       break;
   }
 }
@@ -322,8 +321,7 @@ static void low_push_safearray(SAFEARRAY *psa, UINT dims,
     /* TODO: Handle more array types */
     Pike_error("Unknown vartype: %d\n", vtype);
     /* Not reached! */
-    push_int(0);
-    Pike_sp[-1].subtype = NUMBER_UNDEFINED;
+    push_undefined();
     return;
   }
 
@@ -458,8 +456,7 @@ static void push_varg(VARIANT *varg)
           }
           else
           {
-            push_int(0);
-            Pike_sp[-1].subtype = NUMBER_UNDEFINED;
+            push_undefined();
           }
         }
         else
@@ -714,20 +711,19 @@ static void f_cval_cast(INT32 args)
 
   if (args < 1)
     Pike_error("cast() called without arguments.\n");
-  if (Pike_sp[-args].type != PIKE_T_STRING)
+  if (TYPEOF(Pike_sp[-args]) != PIKE_T_STRING)
     Pike_error("Bad argument 1 to cast().\n");
 
-  if (!strcmp(Pike_sp[-args].u.string->str, "object")) {
+  if( Pike_sp[-args].u.string == literal_string_string )
+  {
     pop_n_elems(args);
-    push_object(this_object());
+    cval_push_result(0, DISPATCH_PROPERTYGET);
+  }
+  else
+  {
+    pop_n_elems(args);
+    push_undefined();
   }
-
-  
-/*   if (strcmp(Pike_sp[-args].u.string->str, "string")) */
-/*     Pike_error("cast() to other type than string.\n"); */
-
-  pop_n_elems(args);
-  cval_push_result(0, DISPATCH_PROPERTYGET);
 }
 
 /*   pike_add_function("`[]", f_cval_ind, "function(mixed:mixed)", 0); */
@@ -750,7 +746,7 @@ static void f_cval_arrow(INT32 args)
   if (args != 1)
     Pike_error("Bad argument to cval::`->\n");
 
-  if (Pike_sp[-1].type == PIKE_T_STRING &&
+  if (TYPEOF(Pike_sp[-1]) == PIKE_T_STRING &&
       !strncmp(Pike_sp[-1].u.string->str, "_value", Pike_sp[-1].u.string->len))
   {
     pop_n_elems(args);
@@ -759,12 +755,11 @@ static void f_cval_arrow(INT32 args)
   }
 
   cval_push_result(0, DISPATCH_PROPERTYGET);
-  if (Pike_sp[-1].type != PIKE_T_OBJECT ||
+  if (TYPEOF(Pike_sp[-1]) != PIKE_T_OBJECT ||
       Pike_sp[-1].u.object->prog != cobj_program)
   {
     pop_n_elems(args+1);
-    push_int(0);
-    Pike_sp[-1].subtype=NUMBER_UNDEFINED;
+    push_undefined();
     return;
   }
   stack_swap();
@@ -780,7 +775,7 @@ static void f_cval_arrow_assign(INT32 args)
     Pike_error("Bad argument to cval::`->=\n");
 
   cval_push_result(0, DISPATCH_PROPERTYGET);
-  if (Pike_sp[-1].type != PIKE_T_OBJECT ||
+  if (TYPEOF(Pike_sp[-1]) != PIKE_T_OBJECT ||
       Pike_sp[-1].u.object->prog != cobj_program)
   {
     pop_n_elems(args+1);
@@ -834,31 +829,31 @@ static void f_cval__sprintf(INT32 args)
   struct string_builder s;
   
 /*   get_all_args("_sprintf",args,"%i",&x); */
-  if (args < 1 || Pike_sp[-args].type != PIKE_T_INT)
+  if (args < 1 || TYPEOF(Pike_sp[-args]) != PIKE_T_INT)
     Pike_error("Bad argument 1 for Com.cval->_sprintf().\n");
-  if (args < 2 || Pike_sp[1-args].type != PIKE_T_MAPPING)
+  if (args < 2 || TYPEOF(Pike_sp[1-args]) != PIKE_T_MAPPING)
     Pike_error("Bad argument 2 for Com.cval->_sprintf().\n");
   
   push_svalue(&Pike_sp[1-args]);
-  push_constant_text("precision");
+  push_text("precision");
   f_index(2);
-  if (Pike_sp[-1].type != PIKE_T_INT)
+  if (TYPEOF(Pike_sp[-1]) != PIKE_T_INT)
     Pike_error("\"precision\" argument to Com->_sprintf() is not an integer.\n");
-  precision_undecided = ((Pike_sp-1)->subtype != NUMBER_NUMBER);
+  precision_undecided = (SUBTYPEOF(Pike_sp[-1]) != NUMBER_NUMBER);
   precision = (--Pike_sp)->u.integer;
   
   push_svalue(&Pike_sp[1-args]);
-  push_constant_text("width");
+  push_text("width");
   f_index(2);
-  if (Pike_sp[-1].type != PIKE_T_INT)
+  if (TYPEOF(Pike_sp[-1]) != PIKE_T_INT)
     Pike_error("\"width\" argument to Com->_sprintf() is not an integer.\n");
-  width_undecided = ((Pike_sp-1)->subtype != NUMBER_NUMBER);
+  width_undecided = (SUBTYPEOF(Pike_sp[-1]) != NUMBER_NUMBER);
   width = (--Pike_sp)->u.integer;
 
   push_svalue(&Pike_sp[1-args]);
-  push_constant_text("flag_left");
+  push_text("flag_left");
   f_index(2);
-  if (Pike_sp[-1].type != PIKE_T_INT)
+  if (TYPEOF(Pike_sp[-1]) != PIKE_T_INT)
     Pike_error("\"flag_left\" argument to Com->_sprintf() is not an integer.\n");
   flag_left=Pike_sp[-1].u.integer;
   pop_stack();
@@ -933,7 +928,7 @@ static void f_cobj_create(INT32 args)
 
   if (args > 0)
   {
-    get_all_args("Com.obj->create()", args, "%W", &progID);
+    get_all_args("create", args, "%W", &progID);
     progID2 = MKPCHARP(malloc(progID->len * 2 + 2), 1);
     pike_string_cpy(progID2, progID);
     SET_INDEX_PCHARP(progID2, progID->len, 0);
@@ -999,7 +994,7 @@ static void f_cobj_getprop(INT32 args)
   struct pike_string *prop;
   PCHARP propU;
 
-  get_all_args("Com.obj->get_prop()", args, "%W", &prop);
+  get_all_args("get_prop", args, "%W", &prop);
   propU = MKPCHARP(malloc(prop->len * 2 + 2), 1);
   pike_string_cpy(propU, prop);
   SET_INDEX_PCHARP(propU, prop->len, 0);
@@ -1043,7 +1038,7 @@ static void f_cobj_setprop(INT32 args)
   if (args!=2)
     Pike_error("Incorrect number of arguments to set_prop.\n");
 
-  if (Pike_sp[-args].type != PIKE_T_STRING)
+  if (TYPEOF(Pike_sp[-args]) != PIKE_T_STRING)
     Pike_error("Bad argument 1 to set_prop.\n");
 
   prop = Pike_sp[-args].u.string;
@@ -1091,7 +1086,7 @@ static void f_cobj_call_method(INT32 args)
   if (args<1)
     Pike_error("Incorrect number of arguments to call_method.\n");
 
-  if (Pike_sp[-args].type != PIKE_T_STRING)
+  if (TYPEOF(Pike_sp[-args]) != PIKE_T_STRING)
     Pike_error("Bad argument 1 to call_method.\n");
 
   prop = Pike_sp[-args].u.string;
@@ -1130,7 +1125,7 @@ static void f_cobj_arrow(INT32 args)
   struct cobj_storage *cobj = THIS_COBJ;
   struct svalue *cval_prog;
 
-  if (args<1 || Pike_sp[-args].type != PIKE_T_STRING ||
+  if (args<1 || TYPEOF(Pike_sp[-args]) != PIKE_T_STRING ||
      args>1)
     Pike_error("Bad args to `->.");
 
@@ -1198,9 +1193,9 @@ static void f_cobj__sprintf(INT32 args)
 {
   struct cobj_storage *cobj = THIS_COBJ;
 
-  if (args < 1 || Pike_sp[-args].type != PIKE_T_INT)
+  if (args < 1 || TYPEOF(Pike_sp[-args]) != PIKE_T_INT)
     Pike_error("Bad argument 1 for Com.cobj->_sprintf().\n");
-  if (args < 2 || Pike_sp[1-args].type != PIKE_T_MAPPING)
+  if (args < 2 || TYPEOF(Pike_sp[1-args]) != PIKE_T_MAPPING)
     Pike_error("Bad argument 2 for Com.cobj->_sprintf().\n");
   
   switch (Pike_sp[-args].u.integer)
@@ -1417,8 +1412,7 @@ static void f_create_object(INT32 args)
   PCHARP progID2;
   IDispatch *pDispatch;
 
-  get_all_args("Com->create_object()", args, "%W", &progID);
-  //check_all_args("create_object", args, BIT_STRING, 0);
+  get_all_args("create_object", args, "%W", &progID);
 
   progID2 = MKPCHARP(malloc(progID->len * 2 + 2), 1);
   pike_string_cpy(progID2, progID);
@@ -1479,7 +1473,7 @@ static void f_get_object(INT32 args)
     PCHARP     tmp;
     type += 0x1;
 
-    if (Pike_sp[-args].type != PIKE_T_STRING)
+    if (TYPEOF(Pike_sp[-args]) != PIKE_T_STRING)
       Pike_error("Bad argument 1 to GetObject\n");
     filename = Pike_sp[-args].u.string;
 
@@ -1494,7 +1488,7 @@ static void f_get_object(INT32 args)
   {
     type += 0x2;
 
-    if (Pike_sp[1-args].type != PIKE_T_STRING)
+    if (TYPEOF(Pike_sp[1-args]) != PIKE_T_STRING)
       Pike_error("Bad argument 2 to GetObject\n");
     progID = Pike_sp[1-args].u.string;
 
@@ -1753,7 +1747,7 @@ static void f_get_typeinfo(INT32 args)
 
   if (args != 1)
     Pike_error("Bad number of arguments for Com->GetTypeInfo().\n");
-  if (Pike_sp[-args].type != PIKE_T_OBJECT ||
+  if (TYPEOF(Pike_sp[-args]) != PIKE_T_OBJECT ||
       Pike_sp[-args].u.object->prog != cobj_program)
     Pike_error("Bad argument 1 for Com->GetTypeInfo().\n");
  
@@ -1797,10 +1791,10 @@ static void f_get_constants(INT32 args)
 
   if (args != 1)
     Pike_error("Bad number of arguments for Com->GetConstants().\n");
-/*   if (Pike_sp[-args].type != PIKE_T_OBJECT || */
+/*   if (TYPEOF(Pike_sp[-args]) != PIKE_T_OBJECT || */
 /*       Pike_sp[-args].u.object->prog != cobj_program) */
 /*     Pike_error("Bad argument 1 for Com->GetConstants().\n"); */
-  if (Pike_sp[-args].type == PIKE_T_STRING)
+  if (TYPEOF(Pike_sp[-args]) == PIKE_T_STRING)
   {
     /* String */
     char *to_free = NULL;
@@ -1821,7 +1815,7 @@ static void f_get_constants(INT32 args)
       return;
     }
   }
-  else if (Pike_sp[-args].type == PIKE_T_OBJECT &&
+  else if (TYPEOF(Pike_sp[-args]) == PIKE_T_OBJECT &&
       Pike_sp[-args].u.object->prog == cobj_program)
   {
     /* cobj */
@@ -1888,15 +1882,15 @@ static void f_com__sprintf(INT32 args)
   struct com_storage *com = THIS_COM;
 #endif /* USE_COM_PROG */
   
-  if (args < 1 || Pike_sp[-args].type != PIKE_T_INT)
+  if (args < 1 || TYPEOF(Pike_sp[-args]) != PIKE_T_INT)
     Pike_error("Bad argument 1 for Com->_sprintf().\n");
-  if (args < 2 || Pike_sp[1-args].type != PIKE_T_MAPPING)
+  if (args < 2 || TYPEOF(Pike_sp[1-args]) != PIKE_T_MAPPING)
     Pike_error("Bad argument 2 for Com->_sprintf().\n");
   
   switch (Pike_sp[-args].u.integer)
   {
     case 'O':
-      push_constant_text("Com()");
+      push_text("Com()");
       stack_pop_n_elems_keep_top(args);
       return;
   }
@@ -1911,8 +1905,7 @@ PIKE_MODULE_INIT
 {
 #ifdef HAVE_COM
   struct svalue prog;
-  prog.type = PIKE_T_PROGRAM;
-  prog.subtype = 0;
+  SET_SVAL(prog, PIKE_T_PROGRAM, 0, program, NULL);
 
   /* load and initialize COM */
   CoInitialize(0);
