@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: eacc5bf5c5eb33fc86d7e520964d651cf8ea9536 $
 */
 
 #ifndef PROGRAM_H
@@ -16,7 +15,7 @@
 #include "time_stuff.h"
 #include "program_id.h"
 #include "pike_rusage.h"
-#include "block_alloc_h.h"
+#include "block_allocator.h"
 
 /* Needed to support dynamic loading on NT */
 PMOD_EXPORT extern struct program_state * Pike_compiler;
@@ -51,6 +50,7 @@ PMOD_EXPORT extern struct program_state * Pike_compiler;
   } while(0)
 
 extern struct pike_string *this_program_string;
+extern struct pike_string *this_string;
 
 /* Common compiler subsystems */
 extern struct pike_string *parser_system_string;
@@ -105,9 +105,13 @@ extern struct pike_string *type_check_system_string;
 #define NUM_LFUNS 45
 
 /* NOTE: After this point there are only fake lfuns.
- *       ie use low_find_lfun(), and NOT FIND_LFUN()!
  */
 #define LFUN__SEARCH 45
+#define LFUN__TYPES 46
+#define LFUN__SERIALIZE 47
+#define LFUN__DESERIALIZE 48
+#define LFUN__SIZE_OBJECT 49
+#define LFUN__RANDOM 50
 
 extern const char *const lfun_names[];
 
@@ -198,6 +202,19 @@ struct object;
 
 union idptr
 {
+  /* For variables: Offset of the variable in the storage pointed to
+   * by inherit.storage_offset in the struct inherit that corresponds
+   * to the identifier. See LOW_GET_GLOBAL and GET_GLOBAL. The stored
+   * variable may be either a normal or a short svalue, depending on
+   * identifier.run_time_type. (IDENTIFIER_VARIABLE)
+   *
+   * For pike functions: Offset to the start of the function in
+   * program.program in the program pointed to by prog in the struct
+   * inherit that corresponds to the identifier. Or -1 if a prototype.
+   * (IDENTIFIER_PIKE_FUNCTION)
+   */
+  ptrdiff_t offset;
+
   /* External symbol reference. (IDENTIFIER_EXTERN)
    *
    * Note that this bit MUST be checked to be zero
@@ -208,6 +225,16 @@ union idptr
     unsigned short id;		/* Reference number. */
   } ext_ref;
 
+  /* Constant. (IDENTIFIER_CONSTANT)
+   *
+   * Offset of the struct program_constant in program.constants
+   * in the program pointed to by prog in the struct inherit
+   * that corresponds to the identifier.
+   */
+  struct {
+    ptrdiff_t offset;	/* Offset in the constants table. */
+  } const_info;
+
   /* Getter/setter reference pair. (IDENTIFIER_VARIABLE && PIKE_T_GET_SET)
    */
   struct {
@@ -218,22 +245,8 @@ union idptr
   /* C function pointer. (IDENTIFIER_C_FUNCTION) */
   void (*c_fun)(INT32);
 
-  /* For variables: Offset of the variable in the storage pointed to
-   * by inherit.storage_offset in the struct inherit that corresponds
-   * to the identifier. See LOW_GET_GLOBAL and GET_GLOBAL. The stored
-   * variable may be either a normal or a short svalue, depending on
-   * identifier.run_time_type. (IDENTIFIER_VARIABLE)
-   *
-   * For constants: Offset of the struct program_constant in
-   * program.constants in the program pointed to by prog in the struct
-   * inherit that corresponds to the identifier. (IDENTIFIER_CONSTANT)
-   *
-   * For pike functions: Offset to the start of the function in
-   * program.program in the program pointed to by prog in the struct
-   * inherit that corresponds to the identifier. Or -1 if a prototype.
-   * (IDENTIFIER_PIKE_FUNCTION)
-   */
-  ptrdiff_t offset;
+  /* Direct svalue pointer. Only used in the reference vtable cache. */
+  struct svalue *sval;
 };
 
 #define IDENTIFIER_VARIABLE 0
@@ -271,7 +284,7 @@ struct identifier
   struct pike_string *name;
   struct pike_type *type;
   unsigned INT32 filename_strno;	/* Index in strings. */
-  unsigned INT32 linenumber;
+  INT_TYPE linenumber;
   unsigned INT8 identifier_flags;	/* IDENTIFIER_??? */
   unsigned INT8 run_time_type;		/* PIKE_T_??? */
   unsigned INT16 opt_flags;		/* OPT_??? */
@@ -352,6 +365,17 @@ struct reference
 
   /* ID_* flags - protected, private etc.. */
   unsigned INT16 id_flags;
+
+  /* V-table cache information from this point on. */
+
+  /* Run-time type for the value field if initialized.
+   *   PIKE_T_UNKNOWN indicates uninitialized.
+   */
+  INT16 run_time_type;
+
+  /* Cached value of the lookup.
+   */
+  union idptr func;
 };
 
 /* Magic value used as identifier reference integer to refer to this. */
@@ -382,7 +406,7 @@ struct inherit
    * until one is found with an inherit_level less than this one. */
   INT16 inherit_level;
 
-  /* All the identifier references in the inherited program has been
+  /* All the identifier references in the inherited program have been
    * copied to this program with the first one at this offset. */
   INT16 identifier_level;
 
@@ -543,9 +567,8 @@ struct pike_trampoline
  * module. */
 #define PROGRAM_LIVE_OBJ 0x2000
 
-/* Indicates that the class is a facet or product_class. */
-#define PROGRAM_IS_FACET 0x4000
-#define PROGRAM_IS_PRODUCT 0x8000
+/* Clear the object storage on destruct. */
+#define PROGRAM_CLEAR_STORAGE	0x4000
 
 /* Using define instead of enum allows for ifdefs - Hubbe */
 #define PROG_EVENT_INIT 0
@@ -621,17 +644,11 @@ struct program
 #include "program_areas.h"
   
   INT16 lfuns[NUM_LFUNS];
-
-#ifdef WITH_FACETS
-  /* Facet related stuff */
-  INT32 facet_index;   /* Index to the facet this facet class belongs to */
-  struct object *facet_group;
-#endif
 };
 
-void dump_program_tables (const struct program *p, int indent);
+PMOD_EXPORT void dump_program_tables (const struct program *p, int indent);
 #ifdef PIKE_DEBUG
-static INLINE int CHECK_IDREF_RANGE (int x, const struct program *p)
+static INLINE int ATTRIBUTE((unused)) CHECK_IDREF_RANGE (int x, const struct program *p)
 {
   if (x < 0 || x >= p->num_identifier_references) {
     dump_program_tables(p, 4);
@@ -658,7 +675,7 @@ static INLINE int CHECK_IDREF_RANGE (int x, const struct program *p)
 
 #ifdef DO_PIKE_CLEANUP
 PMOD_EXPORT extern int gc_external_refs_zapped;
-PMOD_EXPORT void gc_check_zapped (void *a, TYPE_T type, const char *file, int line);
+PMOD_EXPORT void gc_check_zapped (void *a, TYPE_T type, const char *file, INT_TYPE line);
 #endif
 
 #if defined (USE_DLL) && defined (DYNAMIC_MODULE)
@@ -678,7 +695,11 @@ PMOD_EXPORT void gc_check_zapped (void *a, TYPE_T type, const char *file, int li
   }while(0)
 #endif
 
-BLOCK_ALLOC_FILL_PAGES(program, n/a);
+ATTRIBUTE((malloc))
+PMOD_EXPORT struct program * alloc_program(void);
+PMOD_EXPORT void really_free_program(struct program * p);
+void count_memory_in_programs(size_t *num, size_t *_size);
+void free_all_program_blocks(void);
 
 
 extern struct program *first_program;
@@ -752,6 +773,8 @@ struct Supporter
 };
 
 /* Prototypes begin here */
+PMOD_EXPORT void lock_pike_compiler(void);
+PMOD_EXPORT void unlock_pike_compiler(void);
 PMOD_EXPORT void do_free_program (struct program *p);
 void ins_int(INT32 i, void (*func)(char tmp));
 void ins_short(int i, void (*func)(char tmp));
@@ -761,6 +784,7 @@ void unuse_modules(INT32 howmany);
 node *find_module_identifier(struct pike_string *ident,
 			     int see_inherit);
 node *resolve_identifier(struct pike_string *ident);
+PMOD_EXPORT struct program *resolve_program(struct pike_string *ident);
 node *program_magic_identifier (struct program_state *state,
 				int state_depth, int inherit_num,
 				struct pike_string *ident,
@@ -771,8 +795,9 @@ void optimize_program(struct program *p);
 void fsort_program_identifier_index(unsigned short *start,
 				    unsigned short *end,
 				    struct program *p);
-struct pike_string *find_program_name(struct program *p, INT32 *line);
-int override_identifier (struct reference *ref, struct pike_string *name);
+struct pike_string *find_program_name(struct program *p, INT_TYPE *line);
+int override_identifier (struct reference *ref, struct pike_string *name,
+			 int required_flags);
 void fixate_program(void);
 struct program *low_allocate_program(void);
 void low_start_new_program(struct program *p,
@@ -780,10 +805,12 @@ void low_start_new_program(struct program *p,
 			   struct pike_string *name,
 			   int flags,
 			   int *idp);
-PMOD_EXPORT void debug_start_new_program(int line, const char *file);
+PMOD_EXPORT void debug_start_new_program(INT_TYPE line, const char *file);
 void dump_program_desc(struct program *p);
 int sizeof_variable(int run_time_type);
 void check_program(struct program *p);
+int low_is_variant_dispatcher(struct identifier *id);
+int is_variant_dispatcher(struct program *prog, int fun);
 struct program *end_first_pass(int finish);
 PMOD_EXPORT struct program *debug_end_program(void);
 PMOD_EXPORT size_t low_add_storage(size_t size, size_t alignment,
@@ -794,17 +821,24 @@ PMOD_EXPORT void set_gc_recurse_callback(void (*m)(struct object *));
 PMOD_EXPORT void set_gc_check_callback(void (*m)(struct object *));
 PMOD_EXPORT void pike_set_prog_event_callback(void (*cb)(int));
 PMOD_EXPORT void pike_set_prog_optimize_callback(node *(*opt)(node *));
-int really_low_reference_inherited_identifier(struct program_state *q,
-					      int e,
-					      int i);
-int low_reference_inherited_identifier(struct program_state *q,
-				       int e,
-				       struct pike_string *name,
-				       int flags);
+PMOD_EXPORT int really_low_reference_inherited_identifier(struct program_state *q,
+							  int i,
+							  int f);
+PMOD_EXPORT int low_reference_inherited_identifier(struct program_state *q,
+						   int e,
+						   struct pike_string *name,
+						   int flags);
 int find_inherit(struct program *p, struct pike_string *name);
-node *reference_inherited_identifier(struct pike_string *super_name,
-				     struct pike_string *function_name);
+PMOD_EXPORT int reference_inherited_identifier(struct program_state *state,
+					       struct pike_string *inherit,
+					       struct pike_string *name);
 void rename_last_inherit(struct pike_string *n);
+void lower_inherit(struct program *p,
+		   struct object *parent,
+		   int parent_identifier,
+		   int parent_offset,
+		   INT32 flags,
+		   struct pike_string *name);
 PMOD_EXPORT void low_inherit(struct program *p,
 			     struct object *parent,
 			     int parent_identifier,
@@ -856,6 +890,9 @@ PMOD_EXPORT int simple_add_constant(const char *name,
 PMOD_EXPORT int add_integer_constant(const char *name,
 				     INT_ARG_TYPE i,
 				     INT32 flags);
+PMOD_EXPORT int low_add_integer_constant(struct pike_string *name,
+					   INT_ARG_TYPE i,
+					   INT32 flags);
 PMOD_EXPORT int quick_add_integer_constant(const char *name,
 					   int name_length,
 					   INT_ARG_TYPE i,
@@ -884,9 +921,14 @@ INT32 define_function(struct pike_string *name,
 		      unsigned function_flags,
 		      union idptr *func,
 		      unsigned opt_flags);
-int really_low_find_shared_string_identifier(struct pike_string *name,
-					     struct program *prog,
-					     int flags);
+PMOD_EXPORT int really_low_find_shared_string_identifier(struct pike_string *name,
+							 struct program *prog,
+							 int flags);
+int really_low_find_variant_identifier(struct pike_string *name,
+				       struct program *prog,
+				       struct pike_type *type,
+				       int start_pos,
+				       int flags);
 PMOD_EXPORT int low_find_lfun(struct program *p, ptrdiff_t lfun);
 PMOD_EXPORT int find_lfun_fatal(struct program *p, ptrdiff_t lfun);
 int lfun_lookup_id(struct pike_string *lfun_name);
@@ -902,34 +944,43 @@ int store_constant(const struct svalue *foo,
 		   struct pike_string *constant_name);
 struct array *program_indices(struct program *p);
 struct array *program_values(struct program *p);
+struct array *program_types(struct program *p);
+int low_program_index_no_free(struct svalue *to, struct program *p, int e,
+			      struct object *parent, int parent_identifier);
 int program_index_no_free(struct svalue *to, struct svalue *what,
 			  struct svalue *ind);
-int get_small_number(char **q);
-void ext_store_program_line (struct program *prog, INT32 line, struct pike_string *file);
+INT_TYPE get_small_number(char **q);
+void ext_store_program_line (struct program *prog, INT_TYPE line,
+			     struct pike_string *file);
 void start_line_numbering(void);
-void store_linenumber(INT32 current_line, struct pike_string *current_file);
+void store_linenumber(INT_TYPE current_line, struct pike_string *current_file);
 PMOD_EXPORT struct pike_string *low_get_program_line(struct program *prog,
-						     INT32 *linep);
+						     INT_TYPE *linep);
 PMOD_EXPORT struct pike_string *get_program_line(struct program *prog,
-						 INT32 *linep);
-PMOD_EXPORT char *low_get_program_line_plain (struct program *prog, INT32 *linep,
+						 INT_TYPE *linep);
+PMOD_EXPORT char *low_get_program_line_plain (struct program *prog,
+					      INT_TYPE *linep,
 					      int malloced);
 PMOD_EXPORT struct pike_string *low_get_line(PIKE_OPCODE_T *pc,
-					     struct program *prog, INT32 *linep);
+					     struct program *prog,
+					     INT_TYPE *linep);
 PMOD_EXPORT char *low_get_line_plain (PIKE_OPCODE_T *pc, struct program *prog,
-				      INT32 *linep, int malloced);
+				      INT_TYPE *linep, int malloced);
 PMOD_EXPORT struct pike_string *get_line(PIKE_OPCODE_T *pc,
-					 struct program *prog, INT32 *linep);
+					 struct program *prog,
+					 INT_TYPE *linep);
 PMOD_EXPORT struct pike_string *low_get_function_line (struct object *o,
-						       int fun, INT32 *linep);
+						       int fun,
+						       INT_TYPE *linep);
 PMOD_EXPORT struct pike_string *get_identifier_line(struct program *p,
-						    int fun, INT32 *linep);
+						    int fun,
+						    INT_TYPE *linep);
 PMOD_EXPORT void va_yyreport(int severity_level,
-			     struct pike_string *file, INT32 line,
+			     struct pike_string *file, INT_TYPE line,
 			     struct pike_string *system, INT32 args,
 			     const char *fmt, va_list vargs);
 PMOD_EXPORT void low_yyreport(int severity_level,
-			      struct pike_string *file, INT32 line,
+			      struct pike_string *file, INT_TYPE line,
 			      struct pike_string *system,
 			      INT32 args, const char *fmt, ...);
 PMOD_EXPORT void yyreport(int severity_level, struct pike_string *system,
@@ -938,9 +989,9 @@ PMOD_EXPORT void yywarning(char *fmt, ...);
 PMOD_EXPORT void my_yyerror(const char *fmt,...);
 PMOD_EXPORT void yyerror(const char *s);
 void yytype_report(int severity_level,
-		   struct pike_string *expect_file, INT32 expect_line, 
+		   struct pike_string *expect_file, INT_TYPE expect_line,
 		   struct pike_type *expected_t,
-		   struct pike_string *got_file, INT32 got_line,
+		   struct pike_string *got_file, INT_TYPE got_line,
 		   struct pike_type *got_t,
 		   INT32 args, const char *fmt, ...);
 void yytype_error(const char *msg, struct pike_type *expected_t,
@@ -958,7 +1009,8 @@ int call_dependants(struct Supporter *s, int finish);
 int report_compiler_dependency(struct program *p);
 struct compilation;
 void run_pass2(struct compilation *c);
-PMOD_EXPORT void enter_compiler(struct pike_string *filename, int linenumber);
+PMOD_EXPORT void enter_compiler(struct pike_string *filename,
+				INT_TYPE linenumber);
 PMOD_EXPORT void exit_compiler(void);
 struct program *compile(struct pike_string *aprog,
 			struct object *ahandler,
@@ -979,7 +1031,7 @@ void check_all_programs(void);
 void placeholder_index(INT32 args);
 void init_program(void);
 void cleanup_program(void);
-PMOD_EXPORT void visit_program (struct program *p, int action);
+PMOD_EXPORT void visit_program (struct program *p, int action, void *extra);
 void gc_mark_program_as_referenced(struct program *p);
 void real_gc_cycle_check_program(struct program *p, int weak);
 unsigned gc_touch_all_programs(void);
@@ -989,13 +1041,18 @@ void gc_cycle_check_all_programs(void);
 void gc_zap_ext_weak_refs_in_programs(void);
 size_t gc_free_all_unreferenced_programs(void);
 void push_compiler_frame(int lexical_scope);
-void low_pop_local_variables(int level);
-void pop_local_variables(int level);
+node *low_pop_local_variables(int level, node *block);
+node *pop_local_variables(int level, node *block);
 void pop_compiler_frame(void);
+PMOD_EXPORT void *get_inherit_storage(struct object *o, int inherit);
 PMOD_EXPORT ptrdiff_t low_get_storage(struct program *o, struct program *p);
-PMOD_EXPORT char *get_storage(struct object *o, struct program *p);
-struct program *low_program_from_function(struct object *o, INT32 i);
+PMOD_EXPORT void *get_storage(struct object *o, struct program *p);
+PMOD_EXPORT struct program *low_program_from_function(struct object *o, INT32 i);
 PMOD_EXPORT struct program *program_from_function(const struct svalue *f);
+PMOD_EXPORT struct program *program_from_type(const struct pike_type *t);
+PMOD_EXPORT struct program *low_program_from_svalue(const struct svalue *s,
+						    struct object **parent_obj,
+						    int *parent_id);
 PMOD_EXPORT struct program *program_from_svalue(const struct svalue *s);
 struct find_child_cache_s;
 int find_child(struct program *parent, struct program *child);
@@ -1006,21 +1063,29 @@ void yyexplain_not_compatible(int severity_level,
 			      struct program *a, struct program *b);
 void yyexplain_not_implements(int severity_level,
 			      struct program *a, struct program *b);
-PMOD_EXPORT void *parent_storage(int depth);
+void string_builder_explain_not_compatible(struct string_builder *s,
+					   struct program *a,
+					   struct program *b);
+void string_builder_explain_not_implements(struct string_builder *s,
+					   struct program *a,
+					   struct program *b);
+PMOD_EXPORT void *parent_storage(int depth, struct program *expected);
+PMOD_EXPORT void *get_inherited_storage(int inh, struct program *expected);
 PMOD_EXPORT void change_compiler_compatibility(int major, int minor);
 void make_area_executable (char *start, size_t len);
 void make_program_executable(struct program *p);
 /* Prototypes end here */
 
-void count_memory_in_programs(size_t *, size_t *);
-static INLINE int FIND_LFUN(struct program * p, int lfun) {
+static INLINE int ATTRIBUTE((unused)) FIND_LFUN(struct program * p, int lfun) {
+#ifdef PIKE_DEBUG
     dmalloc_touch(struct program*, p);
-    if (lfun < 0 || lfun >= NUM_LFUNS) return find_lfun_fatal(p, lfun);
-
-    if (p->flags & PROGRAM_FIXED) return p->lfuns[lfun];
+    if (lfun < 0) return find_lfun_fatal(p, lfun);
+#endif
+    if (p->flags & PROGRAM_FIXED && lfun < NUM_LFUNS) return p->lfuns[lfun];
     return low_find_lfun(p, lfun);
 }
 
+
 #ifndef PIKE_USE_MACHINE_CODE
 #define make_program_executable(X)
 #endif
@@ -1093,9 +1158,9 @@ static INLINE int FIND_LFUN(struct program * p, int lfun) {
 
 #define start_new_program() debug_start_new_program(__LINE__,__FILE__)
 
-#define visit_program_ref(P, REF_TYPE)				\
+#define visit_program_ref(P, REF_TYPE, EXTRA)			\
   visit_ref (pass_program (P), (REF_TYPE),			\
-	     (visit_thing_fn *) &visit_program, NULL)
+	     (visit_thing_fn *) &visit_program, (EXTRA))
 #define gc_cycle_check_program(X, WEAK) \
   gc_cycle_enqueue((gc_cycle_check_cb *) real_gc_cycle_check_program, (X), (WEAK))
 
