@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: a57d8b9ddd603f84f9f0d4daec09e502107a0cd5 $
 */
 
 #ifndef SSCANF_H
@@ -10,6 +9,10 @@
 
 #define SSCANF_FLAG_76_COMPAT 0x1
 
+INT32 low_sscanf_pcharp(PCHARP input, ptrdiff_t len,
+                        PCHARP format, ptrdiff_t format_len,
+                        ptrdiff_t *chars_matched, int flags);
+
 INT32 low_sscanf(struct pike_string *data, struct pike_string *format, INT32 flags);
 void o_sscanf(INT32 args, INT32 flags);
 PMOD_EXPORT void f_sscanf(INT32 args);
